﻿using Furion;
using Furion.DependencyInjection;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using BPA.Kitchen.Core.Common;
using BPA.KitChen.StoreManagement.SqlSugar;
using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Core.Entity;
using BPA.KitChen.StoreManagement.Core.Entity.Coupon;
using BPA.KitChen.StoreManagement.Core.Enum;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponServices.Dtos;
using BPA.KitChen.StoreManagement.Core.Common;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.FoodMenu.Dtos;
using NetTaste;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponHelperServices.Dtos
{
    public class CouponHelperServices : SupperRepository, ICouponHelperServices, ITransient
    {
        private readonly SqlSugarScope db; // 核心对象：拥有完整的SqlSugar全部功能
        public CouponHelperServices()
        {
            db = SqlSugarDb.Db;
        }



        #region 检查项
        /// <summary>
        /// 检查是否领取过优惠券(判断是否新人)
        /// </summary>
        /// <param name="phone"></param>
        /// <returns></returns>
        public async Task<ResultEntity> IsGetCouponCheck(string memberId)
        {
            var result = new ResultEntity() { IsSuccess = false };

            var data = db.Queryable<BPA_CustomerCoupon, BPA_MemberInfo>((a, b) => new JoinQueryInfos(
                                JoinType.Left, a.Customer_Id == b.Id))
                .Where((a, b) => b.Id == memberId)
                .ToList();
            if (data.Count > 0)
            {
                result.IsSuccess = true;

            }
            else
            {
                result.Msg = "当前用户领取过优惠券";
            }
            return result;
        }

        /// <summary>
        /// 检查优惠券是否可用
        /// </summary>
        /// <param name="couponId"></param>
        /// <returns></returns>
        public async Task<ResultEntity> IsUseCouponCheck(string couponId)
        {
            var result = new ResultEntity() { IsSuccess = false };

            var data = await db.Queryable<BPA_Coupon>()
                .Where(x => x.Id == couponId && x.IsDeleted == 0 && x.ValidStatus == 1 && x.ValidStatus == 1).ToListAsync();
            if (data.Count > 0)
            {
                result.IsSuccess = true;

            }
            else
            {
                result.Msg = "当前优惠券不存在或不可用";
            }
            return result;
        }


        /// <summary>
        /// 检查优惠券用户是否可用
        /// </summary>
        /// <param name="couponId"></param>
        /// <param name="memberId"></param>
        /// <returns></returns>
        public async Task<ResultEntity> IsMemberUseCouponCheck(string couponId,  string memberId)
        {
            var result = new ResultEntity() { IsSuccess = false };

            result = await IsUseCouponCheck(couponId);

            if (result.IsSuccess)
            {
                var customerCoupon = db.Queryable<BPA_CustomerCoupon>()
                    .Where(x => x.IsDeleted == 0 && x.State == 1 && x.Coupon_Id == couponId && x.Customer_Id == memberId && x.Valid_EndTime > DateTime.Now)
                    .ToList();
                if (customerCoupon.Count <= 0)
                {
                    result.IsSuccess = false;
                    result.Msg = "当前用户的优惠券不存在或不可用";
                }
            }

            return result;
        }



        /// <summary>
        ///检查用户是否瞒足优惠券领取数量限制（是否还能在领取此券）
        /// </summary>
        /// <param name="couponId"></param>
        /// <param name="memberId"></param>
        /// <returns></returns>
        public async Task<ResultEntity> IsSatisfyCouponGetLimitCheck(string couponId, string memberId)
        {
            var result = new ResultEntity() { IsSuccess = false };


            //1.查询优惠券
            var coupon = db.Queryable<BPA_Coupon>().First(x => x.Id == couponId);
            if (coupon.GetLimit == 0)
            {
                result.IsSuccess = true;
            }
            else
            {
                //2.查询优惠券用户领取记录
                var customerCoupons = await db.Queryable<BPA_CustomerCoupon>().Where(x => x.Coupon_Id == couponId && x.Customer_Id == memberId).ToListAsync();
                //判断领取记录是否满足优惠券领取限制
                if (customerCoupons.Count < coupon.GetLimit)
                {
                    result.IsSuccess = true;
                }
                else
                {
                    result.Msg = "已达到优惠券最大领取数量";
                }

            }


            return result;
        }

        public async Task<ResultEntity> CouponQuantityCheck(string couponId, string couponRangeId, int count)
        {
            var result = new ResultEntity() { IsSuccess = false };
            var sql = $@"SELECT a.Coupon_Id,a.SendNum,IFNULL( b.GetNum,0 ) AS GetNum from 
                                (SELECT Coupon_Id,sum(SendNum) as SendNum from  BPA_CouponRange WHERE id='{couponRangeId}' GROUP BY Coupon_Id) as a
                                LEFT JOIN(SELECT Coupon_Id, count(*) as GetNum from BPA_CustomerCoupon GROUP BY Coupon_Id )
                                as b on a.Coupon_Id = b.Coupon_Id where a.Coupon_Id in({"'" + couponId + "'"})";
            var data= await db.SqlQueryable<CouponCanNumAndSendNum>(sql).FirstAsync();
            if ((data.SendNum-data.GetNum)<count)
            {
                result.Msg = "优惠券数量不足";
            }
            else
            {

                result.IsSuccess = true;
            }
            return result;
        }

        #endregion

        /// <summary>s
        /// (导入)验证会员优惠券领取限制
        /// </summary>
        /// <param name="couponId"></param>
        /// <param name="listCustomer"></param>
        /// <returns></returns>
        public async Task<MemberInfoOutDto> CouponGetLimitCheck(string couponId, List<MemberInfoDto> memberInfoDtos)
        {
            var notAvailable = new List<MemberInfoDto>();
            var available = new List<MemberInfoDto>();

            var result = await IsUseCouponCheck(couponId);
            if (!result.IsSuccess)
            {
                foreach (var item in memberInfoDtos)
                {
                    notAvailable.Add(new MemberInfoDto()
                    {
                        MemberId = item.MemberId,
                        Phone = item.Phone,
                        Msg = result.Msg
                    });
                }
            }
            else
            {

                //1.查询生效的优惠券信息
                var couponInfo = await db.Queryable<BPA_Coupon>().Where(x => x.Id == couponId && x.ValidStatus == 1).FirstAsync();

                //2.查询会员领取的优惠券
                var customerCoupon = db.Queryable<BPA_CustomerCoupon>()
                    .Where(x => x.Coupon_Id == couponId && memberInfoDtos.Select(y => y.MemberId).Contains(x.Customer_Id))
                    .ToList();

                //3.筛选出没有领取过优惠券的会员
                available = memberInfoDtos.Where(x => !customerCoupon.Select(y => y.Customer_Id).Contains(x.MemberId)).ToList();

                //4.赛选满足条件的会员
                foreach (var item in customerCoupon.GroupBy(x => x.Customer_Id))
                {

                    var phone = memberInfoDtos.FirstOrDefault(x => x.MemberId == item.First().Customer_Id).Phone;
                    //领取数量限制
                    if (item.Count() <= couponInfo.GetLimit)
                    {
                        //优惠券是否过期
                        if (couponInfo.TimeType == 1 && couponInfo.ValidEndTime < DateTime.Now)
                        {
                            notAvailable.Add(new MemberInfoDto()
                            {
                                MemberId = item.First().Customer_Id,
                                Phone = phone,
                                Msg = "优惠券已过期"
                            });
                        }
                        else
                        {
                            available.Add(new MemberInfoDto()
                            {
                                MemberId = item.First().Customer_Id,
                                Phone = phone,
                            });
                        }
                    }
                    else
                    {
                        notAvailable.Add(new MemberInfoDto()
                        {
                            MemberId = item.First().Customer_Id,
                            Phone = phone,
                            Msg = "会员超出领卷数量限制"
                        }); ;
                    }
                }
            }


            return new MemberInfoOutDto()
            {
                Available = available,
                NotAvailable = notAvailable
            };
        }
        /// <summary>
        /// 获取所有可用优惠券（附加剩余数量和使用限制）
        /// </summary>
        /// <returns></returns>
        public async Task<List<CouponCanUseCountAndLimitOutDto>> GetCouponCanUseCountAndLimit(string couponId = null)
        {
            var result = new List<CouponCanUseCountAndLimitOutDto>();
            // 查询出可以可以使用的优惠券
            var couponList = await db.Queryable<BPA_Coupon>().Where(x => x.IsDeleted == 0 && x.ValidStatus == 1)
                .WhereIF(!string.IsNullOrWhiteSpace(couponId), x => x.Id == couponId)
                .ToListAsync();

            var couponListData = new List<BPA_Coupon>();
            //去掉过期优惠券
            foreach (var item in couponList)
            {
                if (item.TimeType == 1 && item.ValidEndTime <= DateTime.Now)
                {
                    couponListData.Add(item);
                }
            }
            couponList = couponList.Where(x => !couponListData.Contains(x)).ToList();
            var couponIds = couponList.Select(x => x.Id).ToList();

            ////查询出优惠券下的所有批次
            var batchNoList = await db.Queryable<BPA_CouponRange>().Where(x => couponIds.Contains(x.Coupon_Id)).ToListAsync();

            //查询所有领取过得优惠券
            var GetCouponList = await db.Queryable<BPA_CustomerCoupon>().Where(x => couponIds.Contains(x.Coupon_Id)).ToListAsync();

            foreach (var item in couponList)
            {
                var list = new List<BatchInfo>();
                //所有发送批次
                var couponRange = batchNoList.Where(x => x.Coupon_Id == item.Id).ToList();

                foreach (var entity in couponRange)
                {
                    //此批的领取情况
                    var GetCount = GetCouponList.Where(x => x.Coupon_Id == item.Id && x.Coupon_Range_Id == entity.Id).ToList();

                    list.Add(new BatchInfo()
                    {
                        Id = entity.Id,
                        BatchNo = entity.BatchNo,
                        SendNum = entity.SendNum,
                        GetNum = GetCount.Count
                    });
                }
                result.Add(new CouponCanUseCountAndLimitOutDto()
                {
                    CouponId = item.Id,
                    CouponName = item.CouponTitle,
                    Limit = item.GetLimit == 0 ? list.Sum(x => x.SendNum) : item.GetLimit,
                    CanUseCount = list.Sum(x => x.SendNum) - list.Sum(x => x.GetNum),
                    BatchInfo = list
                });
            }
            return result;

        }


        /// <summary>
        ///  发送优惠券(同时记录发送记录)
        /// </summary>
        /// <param name="couponId">优惠券</param>
        /// <param name="couponRangeId">批次信息</param>
        /// <param name="memberId">会员</param>
        /// <param name="type">类型</param>
        /// <returns></returns>
        public async Task<ResultEntity> SendCoupon(string couponId, string couponRangeId, int sendCount, string memberId, E_CouponGitType type, SqlSugarScope sqlSugarClient = null)
        {
            if (sqlSugarClient == null)
            {
                sqlSugarClient = db;
            }
            var result = new ResultEntity() { IsSuccess = false };
            var coupon = new BPA_Coupon();

            //检查优惠券数量是否够
            result =await CouponQuantityCheck(couponId, couponRangeId, sendCount);


            //查询会员
            var member = sqlSugarClient.Queryable<BPA_MemberInfo>().Where(x => x.IsDeleted == 0 && x.Status == 0 && x.Id == memberId).ToList().FirstOrDefault();
            if (member != null)
            {
                //2.检查优惠券
                result = await IsUseCouponCheck(couponId);
                if (result.IsSuccess)
                {
                    //查询优惠券
                    coupon = db.Queryable<BPA_Coupon>().First(x => x.Id == couponId);

                    //3.检查用户是是否满足优惠券领取数量限制
                    result = await IsSatisfyCouponGetLimitCheck(couponId, memberId);
                    if (result.IsSuccess)
                    {
                        var list = new List<BPA_CustomerCoupon>();
                        //发券
                        for (int i = 0; i < sendCount; i++)
                        {
                            var temp = new BPA_CustomerCoupon
                            {
                                Customer_Id = memberId,
                                Coupon_Id = couponId,
                                Receive_Type = (int)type,
                                Coupon_Range_Id = couponRangeId,
                                State = 1,
                                CustomerPhone = member.Phone,
                                Valid_StartTime = coupon.TimeType == 1 ? coupon.ValidStartTime : DateTime.Now,
                                Valid_EndTime = coupon.TimeType == 1 ? coupon.ValidEndTime : DateTime.Now.AddDays(coupon.ValidFixedTerm),
                                IsDeleted = 0,
                            };
                            list.Add(temp);
                        }
                        result.IsSuccess = db.Insertable(list).CallEntityMethod(t => t.Create()).ExecuteCommand() > 0; ;
                        if (!result.IsSuccess)
                        {
                            result.Msg = "修改失败";
                        }
                    }
                }
            }
            else
            {
                result.Msg = "会员被停用或不存在此会员";
            }
            return result;
        }


        #region  对外

        /// <summary>
        /// 获取优惠券使用范围
        /// </summary>
        /// <returns></returns>
        public async Task<ResultEntity> GetCouponUseScope(string customerCouponId)
        {
            var result = new ResultEntity() { IsSuccess = true };
            var couponUseScope = new CouponUseScopeOutDto() { };
            var customerCoupon = await db.Queryable<BPA_CustomerCoupon>().Where(x => x.Id == customerCouponId).FirstAsync();
            if (customerCoupon == null)
            {
                result.IsSuccess = false;
                result.Msg = "用户没有此优惠券";
            }
            else if (customerCoupon.State != 1)
            {
                couponUseScope.CouponId = customerCoupon.Coupon_Id;
                result.IsSuccess = false;
                result.Msg = $"优惠券不可以用：{(customerCoupon.State == 2 ? "已核销" : (customerCoupon.State == 3 ? "被占用" : "失效"))}";
            }
            else
            {
                couponUseScope.CouponId = customerCoupon.Coupon_Id;
                var coupon = await db.Queryable<BPA_Coupon>().Where(x => x.Id == customerCoupon.Coupon_Id && x.IsDeleted == 0).FirstAsync();
                if (coupon == null)
                {
                    result.IsSuccess = false;
                    result.Msg = "优惠券不存在";
                }
                else if (coupon.ValidStatus != 1)
                {
                    result.IsSuccess = false;
                    result.Msg = "优惠券被作废或者停用";
                }

                else
                {

                    //批次信息
                    var couponRange = await db.Queryable<BPA_CouponRange>().Where(x => x.Id == customerCoupon.Coupon_Range_Id && x.IsDeleted == 0).FirstAsync();

                    if (couponRange == null)
                    {
                        result.IsSuccess = false;
                        result.Msg = "当前批次信息不可用";
                    }

                    //适用店铺
                    var storeList = new List<UseScopeListOutDto>();

                    var couponRangeRelations = await db.Queryable<BPA_CouponRangeRelations>().Where(x => x.Coupon_Range_Id == couponRange.Id && x.IsDeleted == 0 && x.Status == CommonStatus.ENABLE).ToListAsync();
                    if (couponRangeRelations.Count > 0)
                    {
                        var ids = couponRangeRelations.Select(x => x.RangeTypeId).ToList();
                        if (couponRange.RangeType == 1)
                        {
                            foreach (var item in couponRangeRelations)
                            {
                                storeList.Add(new UseScopeListOutDto()
                                {
                                    Id = item.RangeTypeId
                                });
                            }
                        }
                    }
                    //适用商品
                    var goodsList = new List<UseScopeListOutDto>();
                    var couponGoodsRelations = await db.Queryable<BPA_CouponGoodsRelations>().Where(x => x.Coupon_Range_Id == couponRange.Id && x.IsDeleted == 0 && x.Status == CommonStatus.ENABLE).ToListAsync();
                    if (couponGoodsRelations.Count > 0)
                    {
                        foreach (var item in couponGoodsRelations)
                        {
                            goodsList.Add(new UseScopeListOutDto()
                            {

                                Id = item.GoodsId
                            });
                        }
                    }

                    couponUseScope.StoreList = storeList;
                    couponUseScope.GoodsList = goodsList;

                    result.Data = couponUseScope;
                }
            }

            return result;

        }

        /// <summary>
        /// 查看领取的优惠券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> WacthCoupon(CustomerCouponPageDto input)
        {
            var total = new RefAsync<int>();
            var data = await db.Queryable<BPA_CustomerCoupon, BPA_Coupon>((t, x) => t.Coupon_Id == x.Id)
                .Where(t => t.IsDeleted == 0)
                .Where(t => t.State == input.State)
                //.Where((t, x) => x.ValidStatus == input.State)
                .Where(t => t.Valid_EndTime >= DateTime.Now)
                .Where(t => t.CustomerPhone == input.CustomerPhone || t.Customer_Id == input.CustomerId)
                .Select((t, x) => new CustomerCouponOutputDto
                {
                    CustomerId = t.Customer_Id,
                    CouponTitle = x.CouponTitle,
                    CouponId = t.Coupon_Id,
                    Id = t.Id,
                    State = t.State,
                    CouponRangeId = t.Coupon_Range_Id,
                    OrderId = t.Order_Id,
                    ReceiveType = t.Receive_Type,
                    ValidStartTime = t.Valid_StartTime,
                    ValidEndTime = t.Valid_EndTime,
                    CouponValue = x.CouponValue,
                    Condition = x.Condition
                }).ToPageListAsync(input.Current, input.PageSize, total);
            return new PageUtil
            {
                Data = data,
                Total = total
            };
        }

        /// <summary>
        /// 获取能使用的优惠券
        /// </summary>
        /// <returns></returns>
        public async Task<ResultEntity> GetCanUseCouponByWeChat(WeChatCanUseCouponQueryInputDto inputDto)
        {

            var result = new ResultEntity() { IsSuccess = true };
            //1.查询会员
            var member = db.Queryable<BPA_PlatformMemberInfo>().Where(x => x.PlatformMemberId == inputDto.OpenId).ToList().FirstOrDefault();

            if (member == null)
            {
                result.Msg = "会员不存在";
                return result;
            }
            var StoreGoodss = db.Queryable<ShopEntity>().Where(x => inputDto.StoreGoodsIds.Contains(x.Id)).ToList();

            //店铺
            var storeIds = StoreGoodss.Select(x => x.Id).ToList();


            //商品
            var GoodsIds = db.Queryable<BPA_FoodMenu, BPA_StoreFoodMenu>((a, b) =>
                    new JoinQueryInfos(JoinType.Left, a.Id == b.FoodMenuId))
                .Where((a, b) => storeIds.Contains(b.StoreId))
                .Select((a, b) => a.GoodsId).ToList();




            //2.查询当前会员的所有库用优惠券
            var customerCoupon = db.Queryable<BPA_CustomerCoupon>()
                .Where(x => x.Customer_Id == member.MemberInfo_Id && x.IsDeleted == 0 && x.State == 1 && x.Valid_EndTime > DateTime.Now)
                .ToList();
            var couponRangeId = customerCoupon.Select(x => x.Coupon_Range_Id).ToList();

            //店铺判定
            var batchScopeLimit = db.Queryable<BPA_CouponRangeRelations, BPA_CouponRange>((a, b) => new JoinQueryInfos(JoinType.Left, a.Coupon_Range_Id == b.Id))
                .Where((a, b) => couponRangeId.Contains(a.Coupon_Range_Id))
                .Select((a, b) => new BatchScopeLimitDto()
                {
                    Coupon_Id = b.Coupon_Id,
                    Coupon_Range_Id = a.Coupon_Range_Id,
                    RangeType = b.RangeType,
                    StoreId = b.RangeType == 1 ? a.RangeTypeId : "",
                })
                .ToList();

            var rangeId = new List<string>();
            foreach (var item in batchScopeLimit)
            {
                var coupon_Range_Id = "";
                if (item.RangeType == 1 && storeIds.Contains(item.StoreId))
                {
                    coupon_Range_Id = item.Coupon_Range_Id;
                }

                //商品判定
                var goodsrange = db.Queryable<BPA_CouponGoodsRelations>()
                     .Where(x => x.Coupon_Range_Id == item.Coupon_Range_Id)
                     .ToList();

                if (goodsrange.Count > 0)
                {
                    if (goodsrange.Where(x => GoodsIds.Contains(x.GoodsId)).Select(x => x.Coupon_Range_Id).ToList().Count <= 0)
                    {
                        coupon_Range_Id = "";
                    }
                }
                if (!string.IsNullOrWhiteSpace(coupon_Range_Id))
                {
                    rangeId.Add(coupon_Range_Id);
                }

            }


            var date = db.Queryable<BPA_CustomerCoupon, BPA_Coupon>((a, b) => new JoinQueryInfos(JoinType.Left, a.Coupon_Id == b.Id))
                 .Where((a, b) => rangeId.Contains(a.Coupon_Range_Id) && b.Condition <= inputDto.totalPrice)
                 .Where((a, b) => a.IsDeleted == 0 && a.State == 1 && a.Valid_EndTime > DateTime.Now && b.IsDeleted == 0 && b.ValidStatus == 1)
                 .Where((a, b) => a.Customer_Id == member.MemberInfo_Id)
                 .Select((a, b) => new CustomerCouponOutputDto
                 {
                     Id = a.Id.SelectAll(),
                     Condition = b.Condition,
                     CouponTitle = b.CouponTitle,
                     CouponValue = b.CouponValue,
                     Remark = b.Remarks,
                 }).ToList();
            result.Data = date;
            if (date.Count <= 0)
            {
                result.IsSuccess = false;
            }
            return result;
        }



        /// <summary>
        /// 扫码领券
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<ResultEntity> ScanCodeGetCoupon(ScanCodeGetCouponInputDto inputDto)
        {
            var result = new ResultEntity() { IsSuccess = false };


            var member = await db.Queryable<BPA_MemberInfo>().FirstAsync(x => x.Id == inputDto.MemberId);

            if (member == null)
            {
                result.IsSuccess = false;
                result.Msg = "会员不存在";
                return result;
            }
            //查询优惠券可用批次

            var couponRanges = db.Queryable<BPA_CouponRange>().Where(x => x.IsDeleted == 0 && x.Coupon_Id == inputDto.CouponId).ToList();

            if (couponRanges.Count > 0)
            {
                var groupId = couponRanges.First().GroupId;
                if (groupId != member.GroupId)
                {
                    result.IsSuccess = false;
                    result.Msg = "供应商错误";
                    return result;
                }
            }

            //查询优惠券的使用情况
            var customerCoupons = db.Queryable<BPA_CustomerCoupon>()
                .Where(x => couponRanges.Select(y => y.Coupon_Id).Contains(x.Coupon_Id))
                .GroupBy(x => x.Coupon_Id)
                .Select(x => new { Count = SqlFunc.AggregateCount(x.Coupon_Id), Coupon_Id = x.Coupon_Id }).ToList();

            var data = (from a in couponRanges
                        join b in customerCoupons on a.Coupon_Id equals b.Coupon_Id into ab
                        from abinfo in ab.DefaultIfEmpty()
                        where a.SendNum > (abinfo?.Count ?? 0)
                        select a).ToList();
            if (data.Count <= 0)
            {
                result.IsSuccess = false;
                result.Msg = "没有可用批次";
                return result;
            }
            result.IsSuccess = await App.GetService<ICouponServices>().ReceiveCouponByInput(new CustomerCouponInputDto()
            {
                CouponId = inputDto.CouponId,
                MemberId = new List<string>() { inputDto.MemberId },
                CouponRangeId = data.First().Id,
                CouponGitType = E_CouponGitType.线下门店扫码
            });
            return result;
        }



        #endregion

        #region 优惠券核销

        /// <summary>
        /// 优惠券核销
        /// </summary>
        /// <param name="customerCouponId"></param>
        /// <returns></returns>
        public async Task<ResultEntity> CouponWriteoff(CouponWriteoffInputDto inputDto)
        {
            var result = new ResultEntity() { IsSuccess = false };
            if (string.IsNullOrWhiteSpace(CurrentUser.TenantId))
            {
                result.Msg = "加盟商不能为空";
                return result;
            }
            try
            {
                db.Ado.BeginTran();
                if (inputDto.Type == 1)//第三方优惠券
                {


                    foreach (var item in inputDto.CouponId)
                    {
                        var paperCoupon = await db.Queryable<BPA_PaperCoupon>().FirstAsync(x => x.Id == item);

                        if (paperCoupon == null)
                        {
                            result.Msg = "优惠券不存在";
                            return result;
                        }

                        var res = await db.Insertable(new BPA_PaperCouponWriteoffLog()
                        {
                            CreateAt = DateTime.Now,
                            GroupId = CurrentUser.TenantId,
                            OrderId = inputDto.OrderId,
                            PaperCouponId = item,

                        }).ExecuteCommandAsync();
                        result.IsSuccess = res > 0;
                        if (!result.IsSuccess)
                        {
                            throw new Exception("优惠券核销错误");
                        }
                    }
                }
                else
                {
                    foreach (var item in inputDto.CouponId)
                    {
                        var goodsoupon = await db.Queryable<BPA_GoodsCoupon>().FirstAsync(x => x.Id == item);

                        if (goodsoupon == null)
                        {
                            result.Msg = "优惠券不存在";
                            return result;
                        }

                        var res = await db.Insertable(new BPA_GoodsCouponWriteoffLog()
                        {
                            CreateAt = DateTime.Now,
                            GroupId = CurrentUser.TenantId,
                            OrderId = inputDto.OrderId,
                            GoodsCouponId = item,

                        }).ExecuteCommandAsync();
                        result.IsSuccess = res > 0;
                        if (!result.IsSuccess)
                        {
                            throw new Exception("抵扣券核销错误");
                        }
                    }
                }
                db.Ado.CommitTran();
            }
            catch (Exception ex)
            {
                db.Ado.RollbackTran();
                result.Msg=ex.ToString();
            }
            return result;
        }

        /// <summary>
        ///获取 优惠群核销记录
        /// </summary>
        /// <param name="orderId"></param>
        /// <returns></returns>
        public async Task<ResultEntity> GetPOSCouponWriteoffRecord(string orderId)
        {
            var result = new ResultEntity() { IsSuccess = true };
            var data = new POSCouponWriteoffRecordOutDto();
            data.GoodsCouponWriteoffLogs = await db.Queryable<BPA_GoodsCouponWriteoffLog>().Where(x => x.OrderId == orderId).ToListAsync();
            data.PaperCouponWriteoffLogs = await db.Queryable<BPA_PaperCouponWriteoffLog>().Where(x => x.OrderId == orderId).ToListAsync();
            result.Data = data;
            return result;
        }

        public Task<ResultEntity> SendCoupon(string couponId, string couponRangeId, int sendCount, string memberId, E_CouponGitType type, SqlSugarClient sqlSugarClient = null)
        {
            throw new NotImplementedException();
        }

        #endregion


    }
}
