﻿using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Core.Enum;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponHelperServices.Dtos
{
    public interface ICouponHelperServices
    {

        #region  检查项

        /// <summary>
        /// 检查是否领取过优惠券
        /// </summary>
        /// <param name="phone"></param>
        /// <returns></returns>
        Task<ResultEntity> IsGetCouponCheck(string memberId);

        /// <summary>
        /// 检查优惠券是否可用
        /// </summary>
        /// <param name="couponId"></param>
        /// <returns></returns>
        Task<ResultEntity> IsUseCouponCheck(string couponId);

        /// <summary>
        /// 检查优惠券用户是否可用
        /// </summary>
        /// <param name="couponId"></param>
        /// <param name="memberId"></param>
        /// <returns></returns>
        Task<ResultEntity> IsMemberUseCouponCheck(string couponId, string memberId);

        /// <summary>
        ///检查用户是否瞒足优惠券领取数量限制（是否还能在领取此券）
        /// </summary>
        /// <param name="couponId"></param>
        /// <param name="memberId"></param>
        /// <returns></returns>
        Task<ResultEntity> IsSatisfyCouponGetLimitCheck(string couponId, string memberId);


        #endregion


        /// <summary>
        ///(导入)验证会员优惠券领取限制
        /// </summary>
        /// <param name="CouponId"></param>
        /// <param name="listCustomer"></param>
        /// <returns></returns>
        Task<MemberInfoOutDto> CouponGetLimitCheck(string CouponId, List<MemberInfoDto> memberInfoDtos);


        /// <summary>
        /// 获取所有可用优惠券（附加剩余数量和使用限制）
        /// </summary>
        /// <returns></returns>
        Task<List<CouponCanUseCountAndLimitOutDto>> GetCouponCanUseCountAndLimit(string couponId = null);


        /// <summary>
        ///  发送优惠券(同时记录发送记录)
        /// </summary>
        /// <param name="couponId">优惠券</param>
        /// <param name="couponRangeId">批次信息</param>
        /// <param name="memberId">会员</param>
        /// <param name="type">类型</param>
        /// <returns></returns>
        Task<ResultEntity> SendCoupon(string couponId, string couponRangeId, int sendCount, string memberId, E_CouponGitType type, SqlSugarScope sqlSugarClient = null);


        #region 对外
        /// <summary>
        /// 获取优惠券使用范围
        /// </summary>
        /// <returns></returns>
        Task<ResultEntity> GetCouponUseScope(string customerCouponId);

        /// <summary>
        /// 获取能使用的优惠券
        /// </summary>
        /// <returns></returns>
        Task<ResultEntity> GetCanUseCouponByWeChat(WeChatCanUseCouponQueryInputDto inputDto);

        /// <summary>
        /// 扫码领券
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        Task<ResultEntity> ScanCodeGetCoupon(ScanCodeGetCouponInputDto inputDto);

        #endregion

        #region 优惠券核销

        /// <summary>
        /// 优惠券核销
        /// </summary>
        /// <param name="customerCouponId"></param>
        /// <returns></returns>
        Task<ResultEntity> CouponWriteoff(CouponWriteoffInputDto inputDto);

        /// <summary>
        ///获取 优惠群核销记录
        /// </summary>
        /// <param name="orderId"></param>
        /// <returns></returns>
        Task<ResultEntity> GetPOSCouponWriteoffRecord(string orderId);

        #endregion
    }
}
