﻿
using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponServices.Dtos;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponServices.Dtos.member;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Furion.RemoteRequest;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Components.Forms;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon
{
    /// <summary>
    /// 优惠卷
    /// </summary>
    [ApiDescriptionSettings("优惠券", Tag = "优惠卷管理", SplitCamelCase = false)]
    public class CouponService : IDynamicApiController, ITransient
    {
        readonly ICouponServices _couponServices;
        /// <summary>
        /// 
        /// </summary>
        /// <param name="couponServices"></param>
        public CouponService(ICouponServices couponServices)
        {
            _couponServices = couponServices;

        }

        /// <summary>
        /// 获取商品类型
        /// </summary>
        /// <returns></returns>
        [Get("/api/coupon/goodstype")]
        public async Task<Dictionary<string, string>> GetGoodsType()
        {
            return await _couponServices.GetGoodsType();
        }


        /// <summary>
        /// 获取商品
        /// </summary>
        /// <param name="GoodsTypeId"></param>
        /// <returns></returns>
        [Get("/api/coupon/goods")]
        public async Task<Dictionary<string, string>> GetGoods(string GoodsTypeId)
        {
            return await _couponServices.GetGoods(GoodsTypeId);
        }

        /// <summary>
        /// 获取会员信息
        /// </summary>
        /// <param name="Phone"></param>
        /// <returns></returns>
        [HttpPost("/api/coupon/GetCustomerInfo")]
        public async Task<List<MemberInfoOutputDto>> GetCustomerInfo(CustomerInfoInputDto input)
        {
            return await _couponServices.GetCustomerInfo(input);
        }


        /// <summary>
        /// 获取优惠卷_分页
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<PageUtil> CouponPgage([FromBody] PageCouponInputDto inputDto)
        {
            return await _couponServices.GetCouponPgageAsync(inputDto);
        }

        /// <summary>
        /// 获取优惠卷状态和类型  1.优惠卷状态  2.优惠卷类型
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        [HttpPost("/api/coupon/getCouponStatusOrType")]
        public async Task<List<StatusOrTypeOutDto>> GetCouponStatusOrTypeAsync(int type)
        {
            return await _couponServices.GetCouponStatusOrTypeAsync(type);

        }

        /// <summary>
        /// 新增优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/coupon/addCoupon")]
        public async Task<bool> AddCoupon(CouponBasetDto inputDto)
        {
            return await _couponServices.AddCoupon(inputDto);
        }

        /// <summary>
        /// 删除优惠卷
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        [HttpPost("/api/coupon/deleteCoupon")]
        public async Task<bool> DeleteCoupon(List<string> Ids)
        {
            return await _couponServices.DeleteCoupon(Ids);
        }

        /// <summary>
        /// 修改优惠券状态
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCouponStatus(CouponBasetDto inputDto)
        {
            return await _couponServices.UpdateCouponStatus(inputDto);
        }

        /// <summary>
        /// 修改优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/coupon/updateCoupon")]
        public async Task<bool> UpdateCoupon(CouponBasetDto inputDto)
        {
            return await _couponServices.UpdateCoupon(inputDto);
        }

        /// <summary>
        /// 作废优惠卷
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpPost("/api/coupon/abolishcoupon")]
        public async Task<bool> AbolishCoupon(string Id)
        {
            return await _couponServices.AbolishCoupon(Id);
        }

        #region 优惠券领取
        /// <summary>
        /// 指定客户领取
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPut, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<bool> ReceiveCouponByInput(CustomerCouponInputDto input)
        {
            return await _couponServices.ReceiveCouponByInput(input);
        }
        /// <summary>
        /// 领券分页查看SAAS用
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<PageUtil> GetCustomerPage(CustomerCouponPageExDto input)
        {
            return await _couponServices.GetCustomerPage(input);
        }
        /// <summary>
        /// 失效顾客优惠券
        /// </summary>
        /// <returns></returns>
        [HttpDelete, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<bool> AbolishCustomerCoupon(string Id)
        {
            return await _couponServices.AbolishCustomerCoupon(Id);
        }
        #endregion

        #region 对小程序暴露接口
        /// <summary>
        /// 领券(支持批量领取)
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPut, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<bool> ReceiveCoupon([FromBody] List<CustomerCouponDto> input)
        {
            return await _couponServices.ReceiveCoupon(input);
        }
        /// <summary>
        /// 退单退券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<ResultEntity> RebackCoupon([FromBody] RebackDto input)
        {
            return await _couponServices.RebackCoupon(input.Id);
        }
        /// <summary>
        /// 优惠券的使用
        /// </summary>
        /// <returns>false券无效或者失效或有错误</returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<CouponUseMsgDto> CustomerCouponOff([FromBody] CustomerCouponBaseDto input)
        {
            return await _couponServices.CustomerCouponOff(input);
        }
        /// <summary>
        /// 查看领取的优惠券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<PageUtil> WacthCoupon([FromBody] CustomerCouponPageDto input)
        {
            return await _couponServices.WacthCoupon(input);
        }
        /// <summary>
        /// 回调写入优惠券状态f
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<bool> UpdateState(UpdateStateDto input)
        {
            return await _couponServices.UpdateState(input.OdrerId, input.State);
        }
        /// <summary>
        /// 客户查询领券的详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<CustomerCouponOutputDto> CustomerCouponDetail(string Id)
        {
            return await _couponServices.CustomerCouponDetail(Id);
        }
        /// <summary>
        /// 查询流水日志
        /// </summary>
        /// <returns></returns>
        [HttpGet, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<List<CouponLogOutputDto>> GetCouponLog(string CouponCustomerId)
        {
            return await _couponServices.GetCouponLog(CouponCustomerId);
        }
        /// <summary>
        /// 查询可用券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<List<CustomerCouponOutputDto>> SelectCanUseCoupon(CustomerCouponCanUseDto input)
        {
            return await _couponServices.SelectCanUseCoupon(input);
        }

        /// <summary>
        /// 根据订单号获取用户的优惠券使用状态
        /// </summary>
        /// <param name="OrderId"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false), AllowAnonymous, NonUnify]
        public async Task<List<CusomterCouponDTO>> GetCouponStutasByOrderId(List<string> OrderId)
            => await _couponServices.GetCouponStutasByOrderId(OrderId);
      
        #endregion

        #region 优惠券记录
        /// <summary>
        /// 获取发券的管理记录
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<PageUtil> GetCouponRecordById([FromBody] CouponRecordPageDto input)
        {
            return await _couponServices.GetCouponRecordById(input);
        }
        /// <summary>
        /// 记录汇总
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, AllowAnonymous, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<PageUtil> GetCouponRecordPage(CouponRecordPageDto input)
        {
            return await _couponServices.GetCouponRecordPage(input);
        }
        /// <summary>
        /// 新增发券记录
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPut, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<bool> AddCouponSendRecord([FromBody] CouponRecordDto input)
        {
            return await _couponServices.AddCouponSendRecord(input);
        }
        /// <summary>
        /// 召回优惠券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<bool> RecallCoupon(CouponRecordDto input)
        {
            return await _couponServices.RecallCoupon(input);
        }
        /// <summary>
        /// 召回禁用优惠卷商品
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<bool> RecallCouponGoods(CouponRecordDto input)
        {
            return await _couponServices.RecallCouponGoods(input);
        }
        /// <summary>
        /// 获取发券的管理记录商品
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost, ApiDescriptionSettings(SplitCamelCase = false)]
        public async Task<PageUtil> GetCouponRecordGoods(CouponRecordPageDto input)
        {
            return await _couponServices.GetCouponRecordGoods(input);
        }
        #endregion


        

    }
}
