﻿
using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponHelperServices.Dtos;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponServices.Dtos.member;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.FoodMenu;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.FoodMenu.Dtos;
using BPA.KitChen.StoreManagement.Core.Common;
using BPA.KitChen.StoreManagement.Core.Common.Const;
using BPA.KitChen.StoreManagement.Core.Entity;
using BPA.KitChen.StoreManagement.Core.Entity.Coupon;
using BPA.KitChen.StoreManagement.Core.Enum;
using BPA.KitChen.StoreManagement.SqlSugar;
using Furion;
using Furion.DependencyInjection;
using Furion.FriendlyException;
using Mapster;
using Microsoft.AspNetCore.Components.Forms;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponServices.Dtos
{
    /// <summary>
    /// 优惠卷
    /// </summary>
    public class CouponServices : SupperRepository, ICouponServices, ITransient
    {

        private readonly SqlSugarScope db;
        private readonly ICouponHelperServices _couponHelperServices;
        private readonly IHttpContextAccessor _httpContextAccessor;
        //private readonly IActivityParticipationRecordServices _activityParticipationRecordServices;
        public CouponServices(ICouponHelperServices couponHelperServices,
            IHttpContextAccessor httpContextAccessor
            //IActivityParticipationRecordServices activityParticipationRecordServices
            )
        {
            db = SqlSugarDb.Db;
            _couponHelperServices = couponHelperServices;
            _httpContextAccessor = httpContextAccessor;
            //_activityParticipationRecordServices = activityParticipationRecordServices;
        }

        /// <summary>
        /// 获取优惠卷状态和类型  1.优惠卷状态  2.优惠卷类型
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public async Task<List<StatusOrTypeOutDto>> GetCouponStatusOrTypeAsync(int type)
        {
            var result = new List<StatusOrTypeOutDto>();

            if (type == 1)
            {
                foreach (var item in Enum.GetValues(typeof(E_CouponStatus)))
                {
                    result.Add(new StatusOrTypeOutDto()
                    {
                        Key = (int)item,
                        Value = Enum.GetName(typeof(E_CouponStatus), item)
                    });
                }
            }
            else
            {
                foreach (var item in Enum.GetValues(typeof(E_CouponType)))
                {
                    result.Add(new StatusOrTypeOutDto()
                    {
                        Key = (int)item,
                        Value = Enum.GetName(typeof(E_CouponType), item)

                    });
                }
            }

            return await Task.FromResult(result);
        }

        #region 优惠卷基本操作

        /// <summary>
        /// / 获取优惠卷_分页
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetCouponPgageAsync(PageCouponInputDto inputDto)
        {

            var result = new List<CouponBasetDto>();
            var total = new RefAsync<int>();

            var queryable = db.Queryable<BPA_Coupon>();

            #region 条件查询
            queryable = queryable.Where(x => x.IsDeleted == 0)
                .Where(x => x.CreateType == Convert.ToInt32(App.User.FindFirst(ClaimConst.LoginType).Value))
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.CouponTitle),
                    x => x.CouponTitle.Contains(inputDto.CouponTitle))
                .WhereIF(inputDto.CouponType > 0, x => x.CouponType == inputDto.CouponType)
                .WhereIF(inputDto.CouponValue > 0, x => x.CouponValue == inputDto.CouponValue)
                .WhereIF(inputDto.Condition > 0, x => x.Condition == inputDto.Condition)
                .WhereIF(inputDto.GetLimit > 0, x => x.GetLimit == inputDto.GetLimit)
                .WhereIF(inputDto.TimeType > 0, x => x.TimeType == inputDto.TimeType)
                .WhereIF((inputDto.TimeType == 2 && inputDto.ValidFixedTerm > 0),
                    x => x.ValidFixedTerm == inputDto.ValidFixedTerm)
                .WhereIF(inputDto.TimeType == 1 && inputDto.ValidStartTime.HasValue && inputDto.ValidEndTime.HasValue,
                    x => x.ValidStartTime >= inputDto.ValidStartTime.Value && x.ValidEndTime <= inputDto.ValidEndTime.Value.AddDays(1).AddSeconds(-1))
                .WhereIF(inputDto.ValidStatus > 0, x => x.ValidStatus == inputDto.ValidStatus)
                .WhereIF(inputDto.CreateAt.HasValue, x => x.CreateAt.Date == inputDto.CreateAt.Value.Date);

            #endregion

            var obj=new List<CouponCanNumAndSendNum >();
            //优惠券基础信息
            var data = await queryable.OrderBy(x => x.ValidStatus, OrderByType.Asc).OrderBy(x => x.CreateAt, OrderByType.Desc).ToPageListAsync(inputDto.Current, inputDto.PageSize, total);


            var sql = $@"SELECT a.Coupon_Id,a.SendNum,IFNULL( b.GetNum,0 ) AS GetNum from 
                                (SELECT Coupon_Id,sum(SendNum) as SendNum from  BPA_CouponRange GROUP BY Coupon_Id) as a
                                LEFT JOIN(SELECT Coupon_Id, count(*) as GetNum from BPA_CustomerCoupon GROUP BY Coupon_Id )
                                as b on a.Coupon_Id = b.Coupon_Id where a.Coupon_Id in({string.Join(",", data.Select(x => "'" + x.Id + "'"))})";
            if (data.Count>0)
            {
                obj = await db.SqlQueryable<CouponCanNumAndSendNum>(sql).ToListAsync();
            }

            result = data.Adapt<List<CouponBasetDto>>();
            foreach (var item in result)
            {
                var entity = obj.FirstOrDefault(x => x.Coupon_Id == item.Id);
                if (entity != null)
                {
                    item.SendNum = entity.SendNum;
                    item.GetNum = entity.GetNum;
                    item.IsAllowUpdate = false;

                }
                else
                {
                    item.IsAllowUpdate = true;
                }

            }
            return new PageUtil
            {
                Total = total,
                Data = result
            };
        }

        /// <summary>
        /// 作废优惠卷
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<bool> AbolishCoupon(string Id)
        {


            try
            {
                db.BeginTran();
                var result = false;
                var f1 = true;
                var by = App.User.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
                var couponRangeIdList = db.Queryable<BPA_CouponRange>().Where(x => x.Coupon_Id == Id).Select(x => x.Id).ToList();
                if (couponRangeIdList.Count > 0)
                {
                    f1 = db.Updateable<BPA_CouponRangeRelations>(t => new BPA_CouponRangeRelations() { Status = 0 })
                        .Where(t => couponRangeIdList.Contains(t.Coupon_Range_Id)).ExecuteCommandHasChange();
                    db.Updateable<BPA_CouponGoodsRelations>(t => new BPA_CouponGoodsRelations() { Status = 0 })
                        .Where(t => couponRangeIdList.Contains(t.Coupon_Range_Id)).ExecuteCommandHasChange();
                }
                if (f1)
                {
                    result = db.Updateable<BPA_Coupon>().SetColumns(it =>
                        new BPA_Coupon() { ValidStatus = 3, UpdateAt = DateTime.Now, UpdateBy = by }).Where(it => it.Id == Id).ExecuteCommandHasChange();
                }
                db.CommitTran();
                return result;
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh("作废优惠卷"+ex.Message);
            }
        }


        /// <summary>
        /// 新增优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> AddCoupon(CouponBasetDto inputDto)
        {
            if (inputDto.TimeType == 1 && inputDto.ValidEndTime < inputDto.ValidStartTime)
            {
                throw Oops.Oh("开始时间不能大于结束时间");
            }
            inputDto.Id = Guid.NewGuid().ToString();
            inputDto.CreateAt = DateTime.Now;
            inputDto.CreateBy = App.User.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            inputDto.CreateType = Convert.ToInt32(App.User.FindFirst(ClaimConst.LoginType)?.Value);
            var result = await db.Insertable<BPA_Coupon>(inputDto.Adapt<BPA_Coupon>()).ExecuteCommandAsync();

            return await Task.FromResult(result > 0);
        }

        /// <summary>
        /// 删除优惠卷
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        public async Task<bool> DeleteCoupon(List<string> Ids)
        {

            try
            {
                db.BeginTran();
                var couponRangeIdList = db.Queryable<BPA_CouponRange>().Where(x => Ids.Contains(x.Coupon_Id)).Select(x => x.Id).ToList();
                if (couponRangeIdList.Count > 0)
                {
                    db.Updateable<BPA_CouponRangeRelations>(t => new BPA_CouponRangeRelations() { Status = CommonStatus.DISABLE })
                        .Where(t => couponRangeIdList.Contains(t.Coupon_Range_Id))
                        .ExecuteCommandHasChange();
                    db.Updateable<BPA_CouponGoodsRelations>(t => new BPA_CouponGoodsRelations() { Status = CommonStatus.DISABLE })
                        .Where(t => couponRangeIdList.Contains(t.Coupon_Range_Id))
                        .ExecuteCommandHasChange();
                }
                var deleteBy = App.User.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
                var result =await db.Updateable<BPA_Coupon>().SetColumns(it => new BPA_Coupon
                {
                    IsDeleted = 1,
                    DeleteAt = DateTime.Now,
                    DeleteBy = deleteBy
                }).Where(it => Ids.Contains(it.Id)).ExecuteCommandAsync();
                db.CommitTran();
                return result > 0;
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh(ex.Message);
            }
        }

        /// <summary>
        /// 修改优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCoupon([FromBody] CouponBasetDto inputDto)
        {
            if (inputDto.TimeType == 1 && inputDto.ValidEndTime < inputDto.ValidStartTime)
            {
                throw Oops.Oh("开始时间不能大于结束时间");
            }
            //这个需要改
            var data = db.Queryable<BPA_CouponRange>().Where(x => x.Coupon_Id == inputDto.Id).Any();
            if (!data)
            {
                inputDto.UpdateAt = DateTime.Now;
                inputDto.UpdateBy = App.User.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
                var result = db.Updateable<BPA_Coupon>(inputDto.Adapt<BPA_Coupon>()).IgnoreColumns(it =>
                    new { it.CreateType, it.CreateAt, it.CreateBy, it.DeleteAt, it.DeleteBy, it.IsDeleted }).ExecuteCommand();
                return await Task.FromResult(result > 0);
            }
            else
            {
                throw Oops.Oh($"卷以分发,修改失败");
            }

        }

        /// <summary>
        /// 修改优惠券状态
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCouponStatus([FromBody] CouponBasetDto inputDto)
        {
            var result = await db.Updateable<BPA_Coupon>(inputDto.Adapt<BPA_Coupon>()).UpdateColumns(it =>
                new { it.ValidStatus }).ExecuteCommandAsync();
            return await Task.FromResult(result > 0);

        }


        /// <summary>
        /// 优惠券领取信息（分页）
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetCustomerPage(CustomerCouponPageExDto input)
        {
            var total = new RefAsync<int>();

            var vvv = await db.Queryable<BPA_CustomerCoupon, BPA_Coupon, BPA_CouponRange>((t, x, y) => new JoinQueryInfos(JoinType.Inner, t.Coupon_Id == x.Id, JoinType.Inner, t.Coupon_Range_Id == y.Id)).ToListAsync();


            var res = await db.Queryable<BPA_CustomerCoupon, BPA_Coupon, BPA_CouponRange>((t, x, y) => new JoinQueryInfos(JoinType.Inner, t.Coupon_Id == x.Id, JoinType.Inner, t.Coupon_Range_Id == y.Id))
                .WhereIF(!string.IsNullOrEmpty(input.BatchNo), (t, x, y) => y.BatchNo == input.BatchNo)
                .WhereIF(!string.IsNullOrEmpty(input.CouponTitle), (t, x) => x.CouponTitle.Contains(input.CouponTitle))
                .WhereIF(input.State is < 4, t => t.State == input.State.Value)
                .WhereIF(input.State is 4, t => t.Valid_EndTime < DateTime.Now && t.State == 1)
                .WhereIF(!string.IsNullOrEmpty(input.CustomerPhone), t => t.CustomerPhone.Contains(input.CustomerPhone))
                 .WhereIF(!string.IsNullOrEmpty(input.OrderId), t => t.Order_Id.Contains(input.OrderId))
                .WhereIF(input.ReceiveType.HasValue, t => t.Receive_Type == input.ReceiveType.Value)
                .WhereIF(input.WriteOffTime.HasValue, t => t.WriteOffTime >= input.WriteOffTime.Value && t.WriteOffTime < DateTime.Parse($"{input.WriteOffTime:yyyy-MM-dd 23:59:59}"))
                .WhereIF(input.CreateAt.HasValue, t => t.CreateAt >= input.CreateAt.Value && t.CreateAt < DateTime.Parse($"{input.CreateAt:yyyy-MM-dd 23:59:59}"))
                .Select((t, x, y) => new CustomerCouponPageResultOutputDto
                {
                    CustomerId = t.Customer_Id,
                    Id = t.Id==null?"":t.Id,
                    OrderId = t.Order_Id,
                    ReceiveType = t.Receive_Type,
                    State = t.State,
                    CouponTitle = x.CouponTitle,
                    RangeType = y.RangeType,
                    ValidEndTime = t.Valid_EndTime,
                    ValidStartTime = t.Valid_StartTime,
                    WriteOffTime = t.WriteOffTime,
                    CreateAt = t.CreateAt==null?Convert.ToDateTime("2000-01-01"): t.CreateAt,
                    Phone = t.CustomerPhone
                }).OrderBy(t => t.CreateAt, OrderByType.Desc)
                .ToPageListAsync(input.Current, input.PageSize, total);
            var Ids = res.Select(t => t.Id).ToList();
            var storeIds = db.Queryable<BPA_CouponCustomerStore>().Where(t => Ids.Contains(t.Coupon_Customer_Id)).Select(t => t.StoreId).ToList();
            foreach (var item in res)
            {
                item.StoreName = string.Join(",", db.Queryable<ShopEntity>().Where(t => t.IsDeleted == 0).Where(t => storeIds.Contains(t.Id)).Select(t => t.Name).ToList());


                if (DateTime.Now > item.ValidEndTime && item.State == 1)
                    item.State = 4;
            }
            return new PageUtil { Data = res, Total = total };
        }

        #endregion

        #region 领券

        /// <summary>
        /// 指定客户
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> ReceiveCouponByInput(CustomerCouponInputDto input)
        {

            try
            {
                db.BeginTran();
                foreach (var item in input.MemberId)
                {
                    var result = _couponHelperServices.SendCoupon(input.CouponId, input.CouponRangeId, 1, item, input.CouponGitType.HasValue ? input.CouponGitType.Value : E_CouponGitType.指定领取, db).Result;
                    if (!result.IsSuccess)
                    {
                        throw Oops.Oh(item + ":" + result.Msg);
                    }
                }
                db.CommitTran();
                return true;
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh("发放优惠券到指定客户"+ex.Message);
            }
        }

        /// <summary>
        /// 失效顾客优惠券（作废用户领取的优惠券）
        /// </summary>
        /// <returns></returns>
        public async Task<bool> AbolishCustomerCoupon(string Id)
        {

            try
            {
                db.BeginTran();
                var res= db.Updateable<BPA_CustomerCoupon>()
                    .SetColumns(t => t.State == 4)
                    .Where(t => t.IsDeleted == 0)
                    .Where(t => t.Id == Id)
                    .ExecuteCommandHasChange();
                db.CommitTran();
                return await Task.FromResult(res);
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh(ex.Message);
            }
        }

        #endregion

        #region 对外暴露接口-小程序
        /// <summary>
        /// 领券(支持批量领取)
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> ReceiveCoupon(List<CustomerCouponDto> input)
        {

            try
            {
                db.BeginTran();
                var entity = input.Adapt<List<BPA_CustomerCoupon>>();

                foreach (var item in entity)
                {
                    var result = _couponHelperServices.SendCoupon(item.Coupon_Id, item.Coupon_Range_Id, 1, item.Coupon_Id, E_CouponGitType.指定领取, db).Result;
                    if (!result.IsSuccess)
                    {
                        throw Oops.Oh(item + ":" + result.IsSuccess);
                    }
                };
                db.CommitTran();
                return true;
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh(ex.Message);
            }
        }


        /// <summary>
        /// 退单返还优惠卷
        /// </summary>
        /// <returns></returns>
        public async Task<ResultEntity> RebackCoupon(string OrderId)
        {
            try
            {
                db.BeginTran();
                var customerCoupon = db.Queryable<BPA_CustomerCoupon>()
                    .Where(t => t.Order_Id == OrderId)
                    .Where(t => t.State == 3 || t.State == 2).First();
                if (customerCoupon == null)
                {
                    return new ResultEntity { Msg = "当前订单未使用优惠券", IsSuccess = false };
                }
                customerCoupon.State = 1;

                BPA_CouponLog Log = new BPA_CouponLog
                {
                    Id = Guid.NewGuid().ToString(),
                    CouponId = customerCoupon.Coupon_Id,
                    CustomerId = customerCoupon.Customer_Id,
                    OrderId = OrderId,
                    OrderType = 0,
                    Coupon_Customer_Id = customerCoupon.Id,
                    Remark = $"客户【{customerCoupon.Customer_Id}】退单【{OrderId}】补券【{customerCoupon.Coupon_Id}】类型【退单补券】",
                    OrderTime = DateTime.Now
                };
                db.Insertable(Log).ExecuteCommand();
                var r = db.Updateable(customerCoupon).ExecuteCommandHasChange();
                db.CommitTran();
                return new ResultEntity { Msg = string.Empty, IsSuccess = r };
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh("退单返还优惠卷服务器异常，请检查参数"+ex.Message);
            }
        }


        /// <summary>
        /// 查看领取的优惠券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> WacthCoupon(CustomerCouponPageDto input)
        {
            var total = new RefAsync<int>();
            var data = await db.Queryable<BPA_CustomerCoupon, BPA_Coupon>((t, x) => t.Coupon_Id == x.Id)
                .Where(t => t.IsDeleted == 0)
                 .Where(t => t.State == (input.State.Value==4?1: input.State))
                 .WhereIF(input.State != null && input.State.Value ==4,(t,x)=>t.Valid_EndTime<DateTime.Now)
                .WhereIF(input.State != null && input.State.Value == 3, (t, x) => t.Valid_EndTime > DateTime.Now)
                // .Where((t, x) => x.ValidStatus == 1)
                .Where(t => t.CustomerPhone == input.CustomerPhone || t.Customer_Id == input.CustomerId)
                .Select((t, x) => new CustomerCouponOutputDto
                {
                    CustomerId = t.Customer_Id,
                    CouponTitle = x.CouponTitle,
                    CouponId = t.Coupon_Id,
                    Id = t.Id,
                    State = t.State,
                    CouponRangeId = t.Coupon_Range_Id,
                    OrderId = t.Order_Id,
                    ReceiveType = t.Receive_Type,
                    ValidStartTime = t.Valid_StartTime,
                    ValidEndTime = t.Valid_EndTime,
                    CouponValue = x.CouponValue,
                    Condition = x.Condition
                }).ToPageListAsync(input.Current, input.PageSize, total);
            return new PageUtil
            {
                Data = data,
                Total = total
            };
        }

        /// <summary>
        /// 查询可以用券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<List<CustomerCouponOutputDto>> SelectCanUseCoupon(CustomerCouponCanUseDto input)
        {
            var groupId = App.User?.FindFirst(ClaimConst.GroupId)?.Value;
            StringBuilder sb = new StringBuilder();
            sb.Append($@"SELECT
	                        a.Customer_Id AS CustomerId,
	                        b.CouponTitle,
	                        a.Coupon_Id AS CouponId,
	                        a.Id,
	                        a.State,
	                        a.Coupon_Range_Id AS CouponRangeId,
	                        a.Order_Id AS OrderId,
	                        a.ReceiveType,
	                        b.CouponValue,
	                        b.Condition,
	                       a.ValidStartTime AS ValidStartTime,
	                       a.ValidEndTime AS ValidEndTime,
                           b.Remarks as Remark
                        FROM
	                        BPA_CustomerCoupon AS a
	                        JOIN BPA_Coupon AS b ON a.Coupon_Id= b.Id
	                        JOIN BPA_CouponRange AS c ON a.Coupon_Range_Id= c.Id 
                        WHERE
	                        a.Customer_Id= '{input.CustomerId}' 
                            AND a.State=1 AND b.ValidStatus=1 AND a.IsDeleted=0 and a.ValidEndTime>='{DateTime.Now}'");


            if (input.StoreId.Count() > 0)
                sb.Append($@"  and ( a.Coupon_Range_Id in  (SELECT  Coupon_Range_Id from BPA_CouponRangeRelations WHERE Status=0 and RangeTypeId in 
                                ('{string.Join("','", input.StoreId)}') ))  ");

            if (input.TotalPrice.HasValue)
                sb.Append($@" AND b.Condition<= {input.TotalPrice}");
            var sql = sb.ToString();


            var data = await db.SqlQueryable<CustomerCouponOutputDto>(sql).ToListAsync();


            if (input.GoodsId.Count() > 0)
            {
                //查询有限定食物的批次
                var goodsRelations = db.SqlQueryable<BPA_CouponGoodsRelations>($@"SELECT  * from  BPA_CouponGoodsRelations where Coupon_Range_Id in('{string.Join("','", data.Select(x => x.CouponRangeId).ToList())}') ").ToList();

                //排除满足条件的
                var ids = new List<string>();
                foreach (var item in goodsRelations.GroupBy(x => x.Coupon_Range_Id))
                {
                    var isAdd = true;
                    foreach (var item2 in item)
                    {
                        if (input.GoodsId.Contains(item2.GoodsId))
                        {
                            isAdd = false;
                            break;
                        }
                    }
                    if (isAdd)
                    {
                        ids.Add(item.Key);
                    }
                }

                data = data.Where(x => !ids.Contains(x.CouponRangeId)).ToList();
            }
            return data;
        }


        /// <summary>
        /// 查询领取详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<CustomerCouponOutputDto> CustomerCouponDetail(string Id)
        {
            var data = await db.Queryable<BPA_CustomerCoupon, BPA_Coupon>((t, x) => t.Coupon_Id == x.Id).Where(t => t.Id == Id)
                  .Select((t, x) => new CustomerCouponOutputDto
                  {
                      CustomerId = t.Customer_Id,
                      CouponTitle = x.CouponTitle,
                      CouponId = t.Coupon_Id,
                      Id = t.Id,
                      State = t.State,
                      CouponRangeId = t.Coupon_Range_Id,
                      OrderId = t.Order_Id,
                      ReceiveType = t.Receive_Type,
                      ValidStartTime = t.Valid_StartTime,
                      ValidEndTime = t.Valid_EndTime,
                      Condition = x.Condition,
                      CouponValue = x.CouponValue,
                      WriteOffTime = t.WriteOffTime,
                      Remark = x.Remarks,
                      CouponType = x.CouponType
                  }).FirstAsync();

            if (data == null) return null;
            var RangeType = db.Queryable<BPA_CouponRange>().First(t => t.Id == data.CouponRangeId).RangeType;
            var RangeTypeIds = db.Queryable<BPA_CouponRangeRelations>().Where(t => t.Status == CommonStatus.ENABLE)
                 .Where(t => t.Coupon_Range_Id == data.CouponRangeId).Select(t => t.RangeTypeId).ToList();
            var RangeGoodIds = db.Queryable<BPA_CouponGoodsRelations>().Where(t => t.Status == CommonStatus.ENABLE)
                .Where(t => t.Coupon_Range_Id == data.CouponRangeId).Select(t => t.GoodsId).ToList();
            if (RangeType == 1)
            {
                data.RangeTypeStr = "适用店铺";
                var Names = db.Queryable<ShopEntity>().Where(t => t.IsDeleted == 0).Where(t => RangeTypeIds.Contains(t.Id)).Select(t => t.Name).ToList();
                data.UseRange = Names;
            }
            //if (RangeType == 2)
            //{
            //    data.RangeTypeStr = "适用供应商";
            //    var Names = db.Queryable<BPA_Franchisee>().Where(t => t.Status == 0).Where(t => t.IsDeleted == 0).Where(t => RangeTypeIds.Contains(t.Id)).Select(t => t.Name).ToList();
            //    data.UseRange = Names;
            //}
            //if (RangeType == 3)
            //{
            //    data.RangeTypeStr = "适用区域";
            //    var Names = db.Queryable<BPA_StoreType>().Where(t => t.IsDeleted == 0).Where(t => RangeTypeIds.Contains(t.Id)).Select(t => t.StoreType_Name).ToList();
            //    data.UseRange = Names;
            //}
            if (RangeGoodIds.Count > 0)
            {
                var goodsData =await GetGoodsByMenuIdPage(new FoodMenu.Dtos.FoodMenuQueryInputDto()
                {
                    Current=1,
                    PageSize=999999
                });


                //适用商品
                  data.UseGoods = goodsData.Where(t => RangeGoodIds.Contains(t.id)).Select(t => t.name).ToList();
            }
            data.Remarks = data.Remark?.Split(";").ToList();
            return data;
        }

        /// <summary>
        /// 优惠券的使用
        /// </summary>
        /// <returns>false券无效或者失效或有错误</returns>
        public async Task<CouponUseMsgDto> CustomerCouponOff(CustomerCouponBaseDto input)
        {

            try
            {
                db.BeginTran();


                BPA_CustomerCoupon entity = db.Queryable<BPA_CustomerCoupon, BPA_Coupon>((t, a) => (
                 new JoinQueryInfos(JoinType.Left, t.Coupon_Id == a.Id)))
                   .Where(t => t.Customer_Id == input.CustomerId)
                   .WhereIF(!string.IsNullOrEmpty(input.CouponId), t => t.Coupon_Id == input.CouponId)
                   .Where(t => t.Id == input.CustomerCouponId)
                   .Where(t => t.Valid_StartTime < input.OrderTime && t.Valid_EndTime >= input.OrderTime)
                   .Where((t, a) => t.State == 1 && a.ValidStatus == 1)
                   .Where(t => t.IsDeleted == 0)
                   .First();

                if (entity == null)
                {
                    return new CouponUseMsgDto { Msg = "无效优惠券", IsSuccess = false };
                }
                if (entity.Valid_EndTime <= DateTime.Now)
                {
                    return new CouponUseMsgDto { Msg = "优惠券已过期", IsSuccess = false };
                }


                var data = _couponHelperServices.GetCouponUseScope(input.CustomerCouponId).Result;
                //优惠券范围匹配

                var couponUseScope = data.Data as CouponUseScopeOutDto;

                foreach (var item in input.StoreId)
                {

                    if (!couponUseScope.StoreList.Select(x => x.Id).Contains(item))
                    {
                        return new CouponUseMsgDto { Msg = "使用店铺或区域不匹配", IsSuccess = false };
                    }
                }
                entity.Order_Id = input.OrderId;
                entity.State = (int)input.UseType;
                entity.UpdateAt = DateTime.Now;

                var ret = db.Updateable(entity).ExecuteCommandHasChange();
                List<BPA_CouponCustomerStore> storeLinks = new List<BPA_CouponCustomerStore>();
                input.StoreId.ForEach(t =>
                {
                    BPA_CouponCustomerStore storeLink = new BPA_CouponCustomerStore
                    {
                        Id = Guid.NewGuid().ToString(),
                        Coupon_Customer_Id = entity.Id,
                        StoreId = t
                    };
                    storeLinks.Add(storeLink);
                });
                db.Insertable(storeLinks).ExecuteCommand();
                BPA_CouponLog Log = new BPA_CouponLog
                {
                    Id = Guid.NewGuid().ToString(),
                    CouponId = entity.Coupon_Id,
                    CustomerId = entity.Customer_Id,
                    OrderId = input.OrderId,
                    OrderType = 1,
                    Coupon_Customer_Id = entity.Id,
                    Remark = $"客户【{entity.Customer_Id}】下单【{input.OrderId}】用券【{entity.Coupon_Id}】类型【下单用券】",
                    OrderTime = input.OrderTime
                };
                db.Insertable(Log).ExecuteCommand();
                db.CommitTran();
                return new CouponUseMsgDto { Msg = "", IsSuccess = true };
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh(ex.Message);
            }
        }

        /// <summary>
        /// 回调写入优惠券状态
        /// </summary>
        /// <param name="OdrerId"></param>
        /// <param name="State"></param>
        /// <returns></returns>
        public async Task<bool> UpdateState(string OdrerId, int State)
        {
            try
            {
                db.BeginTran();
                var res= db.Updateable<BPA_CustomerCoupon>().SetColumns(t => t.State == State).Where(t => t.Order_Id == OdrerId).ExecuteCommandHasChange();
                db.CommitTran();
                return await Task.FromResult(res);
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh(ex.Message);
            }
        }

        /// <summary>
        /// 查询流水日志
        /// </summary>
        /// <returns></returns>
        public async Task<List<CouponLogOutputDto>> GetCouponLog(string CouponCustomerId)
        {
            return await db.Queryable<BPA_CouponLog, BPA_CustomerCoupon, BPA_Coupon>((a, b, c) => new JoinQueryInfos(
                JoinType.Inner, a.CouponId == b.Coupon_Id && a.Coupon_Customer_Id == b.Id,
                JoinType.Inner, a.CouponId == c.Id
               )).Where(a => a.Coupon_Customer_Id == CouponCustomerId).Select((a, b, c) => new CouponLogOutputDto
               {
                   Id = a.Id,
                   CouponId = a.CouponId,
                   CouponTitle = c.CouponTitle,
                   CouponType = c.CouponType,
                   CouponValue = c.CouponValue,
                   CustomerId = b.Customer_Id,
                   Phone = b.CustomerPhone,
                   OrderId = a.OrderId,
                   OrderType = a.OrderType,
                   Remark = a.Remark,
                   OrderTime = a.OrderTime
               }).ToListAsync();

        }

        /// <summary>
        /// 根据订单号获取用户的优惠券使用状态
        /// </summary>
        /// <param name="OrderId"></param>
        /// <returns></returns>
        public async Task<List<CusomterCouponDTO>> GetCouponStutasByOrderId(List<string> OrderId)
        {
            var data = await db.Queryable<BPA_CustomerCoupon, BPA_Coupon>((a, b) => a.Coupon_Id == b.Id).Where(a => a.IsDeleted == 0)
              .Where(a => OrderId.Contains(a.Order_Id)).Select((a, b) => new CusomterCouponDTO
              {
                  CouponId = b.Id,
                  CouponTitle = b.CouponTitle,
                  CustomerId = a.Id,
                  OpenId = a.Customer_Id,
                  OrderId = a.Order_Id,
                  Stutas = a.State

              }).ToListAsync();

            return data;
        }

        #endregion

        #region 发券记录(批次发券)
        /// <summary>
        /// 新增发券记录
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> AddCouponSendRecord(CouponRecordDto input)
        {
            var data = await _couponHelperServices.IsUseCouponCheck(input.Coupon_Id);

            if (!data.IsSuccess)
            {
                throw Oops.Oh($"优惠券不可用");
            }

            if (input.RangeType == 1)
            {
                var count = db.Queryable<ShopEntity>().Where(x => input.Relation.Contains(x.Id) && x.IsDeleted == 0 ).ToList();
                if (input.Relation.Count>0&& count.Count()<=0)
                {
                    throw Oops.Oh($"店铺不合法");
                }
            }
            //else if (input.RangeType==3)

            //{
            //    var count = NewSqlSugar.StaticContext().Queryable<BPA_StoreType>().Where(x => input.Relation.Contains(x.Id) && x.IsDeleted == 0 ).ToList();
            //    if (input.Relation.Count > 0 && count.Count() <= 0)
            //    {
            //        throw Oops.Oh($"区域不合法");
            //    }

            //}

            try
            {
                db.BeginTran();
                var entity = input.Adapt<BPA_CouponRange>();
                entity.CreateType = Convert.ToInt32(App.User.FindFirst(ClaimConst.LoginType)?.Value);
                entity.BatchNo = DateTime.Now.ToString("yyyyMMddHHmmssffff");
                var res = db.Insertable(entity).CallEntityMethod(t => t.Create()).ExecuteReturnEntity();
                List<BPA_CouponRangeRelations> temp = new List<BPA_CouponRangeRelations>();
                input.Relation.ForEach(t =>
                {
                    temp.Add(new BPA_CouponRangeRelations
                    {
                        Coupon_Range_Id = res.Id,
                        Status = CommonStatus.ENABLE,
                        RangeTypeId = t
                    });
                });
                List<BPA_CouponGoodsRelations> goods = new List<BPA_CouponGoodsRelations>();
                if (input.GoodsRelation != null && input.GoodsRelation.Count != 0)
                {
                    input.GoodsRelation.ForEach(t =>
                    {
                        goods.Add(new BPA_CouponGoodsRelations
                        {
                            Coupon_Range_Id = res.Id,
                            Status = CommonStatus.ENABLE,
                            GoodsId = t
                        });
                    });
                    db.Insertable(goods).CallEntityMethod(t => t.Create()).ExecuteCommand();
                }
                var res1= db.Insertable(temp).CallEntityMethod(t => t.Create()).ExecuteCommand() > 0; 
                db.CommitTran();
                return await Task.FromResult(res1);
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh(ex.Message);
            }
        }
        /// <summary>
        /// 获取发券的管理记录
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetCouponRecordById(CouponRecordPageDto input)
        {
            var total = new RefAsync<int>();
            var entity = await db.Queryable<BPA_CouponRange, BPA_CouponRangeRelations>((a, b) => a.Id == b.Coupon_Range_Id)
                .Where(a => a.IsDeleted == 0)
                .Where(a => a.BatchNo == input.BatchNo)
                .Where(a => a.Coupon_Id == input.CouponId)
                .WhereIF(input.Status.HasValue && input.Status.Value != 2, (a, b) => b.Status == CommonStatus.ENABLE)
                .WhereIF(input.RangeType.HasValue && input.RangeType.Value != 0, a => a.RangeType == input.RangeType.Value)
                .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select((a, b) => new
                {
                    Coupon_Id = a.Coupon_Id,
                    Id = b.Id,
                    RangeType = a.RangeType,
                    Status = b.Status,
                    Remark = b.Remark,
                    Relat = b.RangeTypeId,
                    SendTime = a.CreateAt==null? "": a.CreateAt.ToString(),
                }).ToPageListAsync(input.Current, input.PageSize, total);

            List<CouponRecordDto> result = new List<CouponRecordDto>();
            foreach (var item in entity)
            {

                CouponRecordDto dto = item.Adapt<CouponRecordDto>();

                dto.Supplier = db.Queryable<ShopEntity>().Where(t => t.IsDeleted == 0).First(t => t.Id == item.Relat).Name;

                result.Add(dto);
            }
            return new PageUtil { Data = result, Total = total };
        }
        /// <summary>
        /// 获取发券的管理记录商品
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetCouponRecordGoods(CouponRecordPageDto input)
        {
            var total = new RefAsync<int>();
            var entity = await db.Queryable<BPA_CouponRange, BPA_CouponGoodsRelations>((a, b) => a.Id == b.Coupon_Range_Id)
                .Where(a => a.IsDeleted == 0)
                .Where(a => a.BatchNo == input.BatchNo)
                .Where(a => a.Coupon_Id == input.CouponId)
                .WhereIF(input.Status.HasValue && input.Status.Value != 2, (a, b) => b.Status == CommonStatus.ENABLE)
                .WhereIF(input.RangeType.HasValue && input.RangeType.Value != 0, a => a.RangeType == input.RangeType.Value)
                .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select((a, b) => new
                {
                    Coupon_Id = a.Coupon_Id,
                    Id = b.Id,
                    RangeType = a.RangeType,
                    Status = b.Status,
                    Remark = b.Remark,
                    Relat = b.GoodsId,
                    SendTime = a.CreateAt
                }).ToPageListAsync(input.Current, input.PageSize, total);

            List<CouponRecordDto> result = new List<CouponRecordDto>();
            foreach (var item in entity)
            {
                var goodsData = await GetGoodsByMenuIdPage(new FoodMenu.Dtos.FoodMenuQueryInputDto()
                {
                    Current = 1,
                    PageSize = 999999
                });


                CouponRecordDto dto = item.Adapt<CouponRecordDto>();
                dto.Supplier = goodsData.First(t => t.id == item.Relat)?.name;
                result.Add(dto);
            }
            return new PageUtil { Data = result, Total = total };
        }
        /// <summary>
        /// 记录汇总
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetCouponRecordPage(CouponRecordPageDto input)
        {
            var total = new RefAsync<int>();

            var data = await db.Queryable<BPA_CouponRange, BPA_CustomerCoupon, BPA_Coupon>((t, x, y) =>
            new JoinQueryInfos(
                JoinType.Left, t.Coupon_Id == x.Coupon_Id && t.Id == x.Coupon_Range_Id,
                JoinType.Left, t.Coupon_Id == y.Id))
               .WhereIF(input.CreateAt.HasValue, t => t.CreateAt.Date == input.CreateAt.Value.Date)
               .WhereIF(input.RangeType.HasValue, t => t.RangeType == input.RangeType.Value)
               .WhereIF(!string.IsNullOrEmpty(input.CouponTitle), (t, x, y) => y.CouponTitle.Contains(input.CouponTitle))
                .WhereIF(!string.IsNullOrEmpty(input.BatchNo), (t, x, y) => t.BatchNo.Contains(input.BatchNo))
                 .OrderBy((t, x, y) => y.ValidStatus)
               .OrderBy((t, x, y) => t.SendNum - SqlFunc.AggregateCount(x.Id), OrderByType.Desc)
               .OrderBy((t, x, y) => t.CreateAt, OrderByType.Desc)
               .GroupBy((t, x, y) => new
               {
                   t.RangeType,
                   t.Id,
                   t.SendNum,
                   t.Coupon_Id,
                   y.CouponTitle,
                   t.CreateAt,
                   t.BatchNo,
                   t.Remark,
                   y.ValidStatus
               }).Select((t, x, y) => new
               {
                   t.RangeType,
                   t.Id,
                   y.CouponTitle,
                   t.SendNum,
                   CreateAt=t.CreateAt==null?"": t.CreateAt.ToString(),
                   t.BatchNo,
                   t.Remark,
                   CouponId = t.Coupon_Id,
                   TotalNum = SqlFunc.AggregateCount(x.Id),
                   LastNum = t.SendNum - SqlFunc.AggregateCount(x.Id),
                   y.ValidStatus
               })
            .ToPageListAsync(input.Current, input.PageSize, total);
            return new PageUtil
            {
                Total = total,
                Data = data
            };
        }
        /// <summary>
        /// 召回禁用优惠卷
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> RecallCoupon(CouponRecordDto input)
        {
            try
            {
                db.BeginTran();
                var res= db.Updateable<BPA_CouponRangeRelations>(t => new BPA_CouponRangeRelations() { Status = (CommonStatus)input.Status, Remark = input.Remark })
                    .Where(t => t.Id == input.Id)
                    .ExecuteCommandHasChange();
                db.CommitTran();
                return await Task.FromResult(res);
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh(ex.Message);
            }
        }

        /// <summary>
        /// 召回禁用优惠卷商品
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<bool> RecallCouponGoods(CouponRecordDto input)
        {
            try
            {
                db.BeginTran();
                var res= db.Updateable<BPA_CouponGoodsRelations>(t => new BPA_CouponGoodsRelations() { Status = CommonStatus.ENABLE, Remark = input.Remark })
                    .Where(t => t.Id == input.Id)
                    .ExecuteCommandHasChange();
                db.CommitTran();
                return await Task.FromResult(res);
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
                throw Oops.Oh(ex.Message);
            }
        }

        #endregion


        /// <summary>
        /// 获取菜品列表
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<List<DataItem>> GetGoodsByMenuIdPage(FoodMenuQueryInputDto inputDto)
        {
            var headers = _httpContextAccessor.HttpContext.Request.Headers;
            var authorization = headers["Authorization"];

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("Authorization", authorization);
            //var url1 = App.Configuration["SAAS_Manage"] + "/api/goods/getgoodslist";
            //var json1 = HttpHelper.HttpGet(url1, "", dic, "");

            var url = App.Configuration["SAAS_Manage"] + "api/goods/page";
            var json2 = HttpHelper.PostData(url, JsonConvert.SerializeObject(new
            {
                current = inputDto.Current,
                pageSize = inputDto.PageSize,
                name = inputDto.Name,

            }), Encoding.UTF8, "application/json", dic);

            var data = JsonConvert.DeserializeObject<BaseResultDto>(json2);
            return data.data.data;
        }

        /// <summary>
        /// 获取商品类型
        /// </summary>
        /// <returns></returns>
        public async Task<List<GoodsTypeDataItem>> GetGoodsTypeList()
        {
            var headers = _httpContextAccessor.HttpContext.Request.Headers;
            var authorization = headers["Authorization"];

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("Authorization", authorization);
            //var url1 = App.Configuration["SAAS_Manage"] + "/api/goods/getgoodslist";
            //var json1 = HttpHelper.HttpGet(url1, "", dic, "");

            var url = App.Configuration["SAAS_Manage"] + "api/goodstype/tree";
            var json2 = HttpHelper.HttpGet(url, "", dic,  "application/json");

            var data = JsonConvert.DeserializeObject<BaseResultGoodsTypeDto>(json2);
            return data.data;
        }


        /// <summary>
        /// 获取商品类型
        /// </summary>
        /// <returns></returns>
        public async Task<Dictionary<string, string>> GetGoodsType()
        {
            var data = await GetGoodsTypeList();

            return data.ToDictionary(t => t.key, t => t.title);
        }

        /// <summary>
        /// 获取商品
        /// </summary>
        /// <param name="GoodsTypeId"></param>
        /// <returns></returns>
        public async Task<Dictionary<string, string>> GetGoods(string GoodsTypeId)
        {
            var goodsData = await GetGoodsByMenuIdPage(new FoodMenu.Dtos.FoodMenuQueryInputDto()
            {
                Current = 1,
                PageSize = 999999
            });

            //商品
            return goodsData.WhereIF(GoodsTypeId != "-1", t => t.goodsTypeId == GoodsTypeId)
                .ToDictionary(t => t.id, t => t.name);
        }

        /// <summary>
        /// 获取会员信息
        /// </summary>
        /// <param name="Phone"></param>
        /// <returns></returns>
        public async Task<List<MemberInfoOutputDto>> GetCustomerInfo(CustomerInfoInputDto input)
        {

            return await db.Queryable<BPA_MemberInfo>().Where(t => !SqlFunc.IsNullOrEmpty(t.Phone))
                .WhereIF(!string.IsNullOrEmpty(input.Phone), t => t.Phone.Contains(input.Phone)).Select(t => new MemberInfoOutputDto
                {
                    Phone = t.Phone,
                    MemberId = t.Id
                }).ToListAsync();


        }
    }
}
