﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponServices.Dtos.member;
using Microsoft.AspNetCore.Mvc;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.CouponServices.Dtos
{

    /// <summary>
    /// 优惠卷接口
    /// </summary>
    public interface ICouponServices
    {

        /// <summary>
        ///获取优惠卷状态和类型  1.优惠卷状态  2.优惠卷类型
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        Task<List<StatusOrTypeOutDto>> GetCouponStatusOrTypeAsync(int type);

        /// <summary>
        /// 新增优惠卷
        /// </summary>
        Task<bool> AddCoupon(CouponBasetDto inputDto);

        /// <summary>
        /// 获取优惠卷_分页
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
         Task<PageUtil> GetCouponPgageAsync(PageCouponInputDto inputDto);

        /// <summary>
        /// 修改优惠卷
        /// </summary>
        Task<bool> UpdateCoupon(CouponBasetDto inputDto);

        /// <summary>
        /// 修改优惠券状态
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        Task<bool> UpdateCouponStatus( CouponBasetDto inputDto);

        /// <summary>
        /// 删除优惠卷
        /// </summary>
        Task<bool> DeleteCoupon(List<string> Ids);

        /// <summary>
        /// 作废
        /// </summary>
        Task<bool> AbolishCoupon(string Id);

        /// <summary>
        /// 领券支持批量领取
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>

        Task<bool> ReceiveCoupon(List<CustomerCouponDto> input);
        /// <summary>
        /// 指定客户领取
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<bool> ReceiveCouponByInput(CustomerCouponInputDto input);
        /// <summary>
        /// 领券分页查看SAAS用
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<PageUtil> GetCustomerPage(CustomerCouponPageExDto input);
        /// <summary>
        /// 退单退券
        /// </summary>
        /// <param name="OrderId"></param>
        /// <returns></returns>
        Task<ResultEntity> RebackCoupon(string OrderId);
        /// <summary>
        /// 优惠券使用状态
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<CouponUseMsgDto> CustomerCouponOff(CustomerCouponBaseDto input);
        /// <summary>
        /// 失效优惠卷
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        Task<bool> AbolishCustomerCoupon(string Id);
        /// <summary>
        /// 查看领取的优惠券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<PageUtil> WacthCoupon(CustomerCouponPageDto input);
        /// <summary>
        /// 回调写入优惠券状态
        /// </summary>
        /// <param name="OdrerId"></param>
        /// <param name="State"></param>
        /// <returns></returns>
        Task<bool> UpdateState(string OdrerId, int State);
        /// <summary>
        /// 查看优惠券流水
        /// </summary>
        /// <param name="couponCustomerId"></param>
        /// <returns></returns>
        Task<List<CouponLogOutputDto>> GetCouponLog(string couponCustomerId);
        /// <summary>
        /// 根据订单号获取用户的优惠券使用状态
        /// </summary>
        /// <param name="OrderId"></param>
        /// <returns></returns>
        Task<List<CusomterCouponDTO>> GetCouponStutasByOrderId(List<string> OrderId);
        /// <summary>
        /// 领券详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        Task<CustomerCouponOutputDto> CustomerCouponDetail(string Id);
        /// <summary>
        /// 新增发券记录
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<bool> AddCouponSendRecord(CouponRecordDto input);
        /// <summary>
        /// 获取发券记录
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<PageUtil> GetCouponRecordById(CouponRecordPageDto input);
        /// <summary>
        /// 记录汇总
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<PageUtil> GetCouponRecordPage(CouponRecordPageDto input);
        /// <summary>
        /// 召回禁用优惠卷
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<bool> RecallCoupon(CouponRecordDto input);
        /// <summary>
        /// 召回禁用优惠卷商品
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<bool> RecallCouponGoods(CouponRecordDto input);
        /// <summary>
        /// 获取发券的管理记录商品
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<PageUtil> GetCouponRecordGoods(CouponRecordPageDto input);
        /// <summary>
        /// 查询可用券
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<List<CustomerCouponOutputDto>> SelectCanUseCoupon(CustomerCouponCanUseDto input);



        /// <summary>
        /// 获取商品类型
        /// </summary>
        /// <returns></returns>
        Task<Dictionary<string, string>> GetGoodsType();

        /// <summary>
        /// 获取商品
        /// </summary>
        /// <param name="GoodsTypeId"></param>
        /// <returns></returns>
         Task<Dictionary<string, string>> GetGoods(string GoodsTypeId);

        /// <summary>
        /// 获取会员信息
        /// </summary>
        /// <param name="Phone"></param>
        /// <returns></returns>
         Task<List<MemberInfoOutputDto>> GetCustomerInfo(CustomerInfoInputDto input);

    }
}
