﻿using Furion.FriendlyException;
using Furion;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using Mapster;
using Furion.DependencyInjection;
using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.GoodsCoupon.Dtos;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.PaperCouponsServices.Dtos;
using BPA.KitChen.StoreManagement.Core.Common;
using BPA.KitChen.StoreManagement.Core.Common.Const;
using BPA.KitChen.StoreManagement.Core.Entity.Coupon;
using BPA.KitChen.StoreManagement.SqlSugar;
using BPA.Models.BPA_Kitchen;
using BPA.KitChen.StoreManagement.Core.Entity;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.GoodsCoupon
{
    public class GoodsCouponServices : SupperRepository, IGoodsCouponServices, ITransient
    {
        
        private readonly SqlSugarScope db; // 核心对象：拥有完整的SqlSugar全部功能

        public GoodsCouponServices()
        {
            db = SqlSugarDb.Db;

        }
        /// <summary>
        /// / 获取优惠卷_分页
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetGoodsCouponPgageAsync(PageGoodsCouponInputDto inputDto)
        {
            var groupId = string.IsNullOrWhiteSpace(CurrentUser.TenantId)
                ? App.User?.FindFirst(ClaimConst.GroupId)?.Value
                : CurrentUser.TenantId;
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商不能为空");
            var result = new List<PaperCouponBasetDto>();
            var total = new RefAsync<int>();
            var queryable = await db.Queryable<BPA_GoodsCoupon, BPA_PaperCouponType, ShopEntity>((a, b,c) =>
                    new JoinQueryInfos(
                    JoinType.Left, a.CTypeId == b.Id,
                    JoinType.Left, a.StoreId == c.Id))
                .Where((a, b,c) => a.GroupId == groupId)
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.CTypeId), a => a.CTypeId == inputDto.CTypeId)
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.Title), a => a.Title.Contains(inputDto.Title))
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.GoodsName), a => a.GoodsName.Contains(inputDto.GoodsName))
                .WhereIF(inputDto.Money > 0, a => a.Money == inputDto.Money)
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.StoreId),a=>a.StoreId== inputDto.StoreId)
                .WhereIF(inputDto.Enable != null, a => a.Enable == inputDto.Enable)
                .Select((a, b,c) => new PaperCouponBasetDto()
                {
                    Id = a.Id,
                    CTypeId = a.CTypeId,
                    Title = a.Title,
                    SoureMoney=a.SoureMoney,
                    GoodsId= a.GoodsId,
                    StoreId= a.StoreId,
                    StoreName=c.Name,
                    Money = a.Money,
                    GoodsName=a.GoodsName,
                    CTypeName = b.Name,
                    CrateTimeAt = a.CrateTimeAt,
                    Enable=a.Enable,
                }).ToPagedListAsync(inputDto.Current, inputDto.PageSize);
            result = queryable.Items.ToList();
            return new PageUtil
            {
                Total = queryable.TotalCount,
                Data = result
            };
        }

        /// <summary>
        /// 新增优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> AddGoodsCoupon(GoodsCouponBasetDto inputDto)
        {
            if (string.IsNullOrWhiteSpace(inputDto.GoodsId))
            {
                throw Oops.Oh("商品不能为空");
            }
            if (string.IsNullOrWhiteSpace(inputDto.Title))
            {
                throw Oops.Oh("标题参数不能为空");
            }
            if (string.IsNullOrWhiteSpace(inputDto.CTypeId))
            {
                throw Oops.Oh("平台参数不能为空");
            }
            if (inputDto.Money == null)
            {
                throw Oops.Oh("原始金额参数不能为空");
            }
            if (inputDto.SoureMoney == null)
            {
                throw Oops.Oh("平台金额参数不能为空");
            }
            var checkMoneyType = db.Queryable<BPA_GoodsCoupon>().Any(x => x.CTypeId == inputDto.CTypeId && x.SoureMoney== inputDto.SoureMoney &&  x.Money == inputDto.Money && x.GoodsId == inputDto.GoodsId && x.StoreId== inputDto.StoreId);
            if (checkMoneyType) throw Oops.Oh("已存在相同平台金额的商品优惠卷");
            inputDto.Id= Guid.NewGuid().ToString();
            inputDto.CrateTimeAt = DateTime.Now;
            inputDto.Enable = true;
            var result = await db.Insertable<BPA_GoodsCoupon>(inputDto.Adapt<BPA_GoodsCoupon>()).ExecuteCommandAsync();

            return await Task.FromResult(result > 0);
        }

        /// <summary>
        /// 更新优惠卷启用状态
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGoodsCouponEnable(string Ids)
        {
            var model = db.Queryable<BPA_GoodsCoupon>().First(x => Ids.Contains(x.Id));
            if (model == null) throw Oops.Oh("数据不存在");
            model.Enable = !model.Enable;
            var result = db.Updateable<BPA_GoodsCoupon>(model).ExecuteCommand();
            return result > 0;
        }

        /// <summary>
        /// 修改优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGoodsCoupon([FromBody] GoodsCouponBasetDto inputDto)
        {
            if (string.IsNullOrWhiteSpace(inputDto.Title))
            {
                throw Oops.Oh("标题参数不能为空");
            }
            if (string.IsNullOrWhiteSpace(inputDto.CTypeId))
            {
                throw Oops.Oh("类型参数不能为空");
            }
            if (inputDto.Money == null)
            {
                throw Oops.Oh("金额参数不能为空");
            }
            if (inputDto.Money < 0)
            {
                throw Oops.Oh("金额不能小于0");
            }
            var result = db.Updateable<BPA_GoodsCoupon>(inputDto.Adapt<BPA_GoodsCoupon>()).UpdateColumns(it =>
                 new { it.Title, it.Money, it.CTypeId,it.GoodsId,it.GoodsName,it.SoureMoney }).ExecuteCommand();
            return await Task.FromResult(result > 0);
        }
        /// <summary>
        /// 统计报表F
        /// </summary>
        /// <param name="paperCouponReportdto"></param>
        /// <returns></returns>
        public async Task<List<GoodsCouponReportDatadto>> GetGoodsCouponReport(PaperCouponReportdto paperCouponReportdto)
        {
            var groupId =  string.IsNullOrWhiteSpace(CurrentUser.TenantId)
               ? App.User?.FindFirst(ClaimConst.GroupId)?.Value
               : CurrentUser.TenantId;
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商不能为空");
            var queryable = await db.Queryable<BPA_GoodsCoupon, BPA_PaperCouponType>((a, b) => a.CTypeId == b.Id)
                .Where((a,b)=>a.GroupId== groupId)
               .WhereIF(!string.IsNullOrWhiteSpace(paperCouponReportdto.CTypeId), a => a.CTypeId == paperCouponReportdto.CTypeId)
               .WhereIF(paperCouponReportdto.Money > 0, a => a.Money == paperCouponReportdto.Money)

               .Select((a, b) => new GoodsCouponReportDatadto()
               {
                   Id = a.Id,
                   CTypeId = a.CTypeId,
                   Title = a.Title,
                   Money = a.Money,
                   SoureMoney= a.SoureMoney,
                   GoodsName= a.GoodsName,
                   CTypeName = b.Name,
                   CrateTimeAt = a.CrateTimeAt
               }).Mapper(p =>
               {
                   p.Allcount = db.Queryable<BPA_GoodsCouponWriteoffLog>()
                    .Where(x => x.GoodsCouponId == p.Id && x.GroupId == groupId)
                   .WhereIF(!string.IsNullOrWhiteSpace(paperCouponReportdto.StartDateStr), c => c.CreateAt >= Convert.ToDateTime(paperCouponReportdto.StartDateStr))
                   .WhereIF(!string.IsNullOrWhiteSpace(paperCouponReportdto.EndDateStr), c => c.CreateAt <= Convert.ToDateTime(paperCouponReportdto.EndDateStr))
                   .Count();
               })
               .ToListAsync();
            return queryable;
        }
    }
}
