﻿using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.GoodsCoupon;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.GoodsCoupon.Dtos;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.PaperCouponsServices.Dtos;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon
{
    /// <summary>
    /// 商品优惠卷管理
    /// </summary>
    [ApiDescriptionSettings("商品优惠券", Tag = "商品优惠卷管理")]
    public class GoodsCouponService : IDynamicApiController, ITransient
    {
        IGoodsCouponServices _goodsCouponServices;
        /// <summary>
        /// 
        /// </summary>
        /// <param name="goodsCouponServices"></param>
        public GoodsCouponService(IGoodsCouponServices goodsCouponServices)
        {
            _goodsCouponServices=goodsCouponServices;
        }
        /// <summary>
        /// 获取优惠卷_分页
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/goodscoupon/getGoodsCouponPgage")]
        [AllowAnonymous]
        public async Task<PageUtil> GetGoodsCouponPgage([FromBody] PageGoodsCouponInputDto inputDto)
        {
            return await _goodsCouponServices.GetGoodsCouponPgageAsync(inputDto);
        }
        /// <summary>
        /// 新增优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/goodscoupon/addGoodsCoupon")]
        public async Task<bool> AddGoodsCoupon(GoodsCouponBasetDto inputDto)
        {
            return await _goodsCouponServices.AddGoodsCoupon(inputDto);
        }

        /// <summary>
        /// 更新优惠卷启用状态
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        [HttpGet("/api/goodscoupon/updateGoodsCouponEnable")]
        public async Task<bool> UpdateGoodsCouponEnable(string Ids)
        {
            return await _goodsCouponServices.UpdateGoodsCouponEnable(Ids);
        }
        /// <summary>
        /// 修改优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/goodscoupon/updateGoodsCoupon")]
        public async Task<bool> UpdateGoodsCoupon(GoodsCouponBasetDto inputDto)
        {
            return await _goodsCouponServices.UpdateGoodsCoupon(inputDto);
        }
        /// <summary>
        /// 优惠卷报表统计
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/goodscoupon/getGoodsCouponReport")]
        [AllowAnonymous]
        public async Task<List<GoodsCouponReportDatadto>> GetPaperCouponReport(PaperCouponReportdto inputDto)
        {
            return await _goodsCouponServices.GetGoodsCouponReport(inputDto);
        }
    }
}
