﻿using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.PaperCouponsServices;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.PaperCouponsServices.Dtos;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.Kitchen.CRM.CouponCenter
{
    [ApiDescriptionSettings("纸质优惠券", Tag = "纸质优惠券管理")]
    public class PaperCouponsService: IDynamicApiController, ITransient
    {
        readonly IPaperCouponsServices _paperCouponsServices;
        /// <summary>
        /// 
        /// </summary>
        /// <param name="couponServices"></param>
        public PaperCouponsService(IPaperCouponsServices paperCouponsServices)
        {
            _paperCouponsServices = paperCouponsServices;

        }
        /// <summary>
        /// 获取优惠卷_分页
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/papercoupon/paperCouponPgage")]
        [AllowAnonymous]
        public async Task<PageUtil> PaperCouponPgage([FromBody] PagePaperCouponInputDto inputDto)
        {
            return await _paperCouponsServices.GetPaperCouponPgageAsync(inputDto);
        }
        /// <summary>
        /// 新增优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/papercoupon/addPaperCoupon")]
        public async Task<bool> AddPaperCoupon(PaperCouponBasetDto inputDto)
        {
            return await _paperCouponsServices.AddPaperCoupon(inputDto);
        }

        /// <summary>
        /// 删除优惠卷
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        [HttpGet("/api/papercoupon/updatePaperCouponEnable")]
        public async Task<bool> UpdatePaperCouponEnable(string Ids)
        {
            return await _paperCouponsServices.UpdatePaperCouponEnable(Ids);
        }
        /// <summary>
        /// 修改优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/papercoupon/updatePaperCoupon")]
        public async Task<bool> UpdateCoupon(PaperCouponBasetDto inputDto)
        {
            return await _paperCouponsServices.UpdatePaperCoupon(inputDto);
        }
        /// <summary>
        /// 新增优惠卷类型
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/papercoupon/addPaperCouponType")]
        public async Task<bool> AddPaperCouponType(PaperCouponTypeBasetDto inputDto)
        {
            return await _paperCouponsServices.AddPaperCouponType(inputDto);
        }
        /// <summary>
        /// 查询所有优惠卷类型
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/papercoupon/getPaperCouponType")]
        [AllowAnonymous]
        public async Task<List<PaperCouponTypeSelectDto>> GetPaperCouponType()
        {
            return await _paperCouponsServices.GetPaperCouponType();
        }
        /// <summary>
        /// 优惠卷报表统计
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/papercoupon/getPaperCouponReport")]
        [AllowAnonymous]
        public async Task<List<PaperCouponReportDatadto>> GetPaperCouponReport(PaperCouponReportdto inputDto)
        {
            return await _paperCouponsServices.GetPaperCouponReport(inputDto);
        }
    }
}
