﻿using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.PaperCouponsServices.Dtos;
using BPA.KitChen.StoreManagement.Core.Common;
using BPA.KitChen.StoreManagement.Core.Common.Const;
using BPA.KitChen.StoreManagement.Core.Entity.Coupon;
using BPA.KitChen.StoreManagement.SqlSugar;
using Furion;
using Furion.DependencyInjection;
using Furion.FriendlyException;
using Mapster;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Coupon.PaperCouponsServices
{
    /// <summary>
    /// 纸质优惠卷服务
    /// </summary>
    public class PaperCouponsServices: SupperRepository, IPaperCouponsServices, ITransient
    {
        private readonly SqlSugarScope db; // 核心对象：拥有完整的SqlSugar全部功能

        public PaperCouponsServices()
        {
            db = SqlSugarDb.Db;

        }
        /// <summary>
        /// / 获取优惠卷_分页
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetPaperCouponPgageAsync(PagePaperCouponInputDto inputDto)
        {
            var groupId = string.IsNullOrWhiteSpace(CurrentUser.TenantId)
                ? App.User?.FindFirst(ClaimConst.GroupId)?.Value
                : CurrentUser.TenantId;
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商不能为空");
            var result = new List<PaperCouponBasetDto>();
            var total = new RefAsync<int>();
            var queryable =await db.Queryable<BPA_PaperCoupon, BPA_PaperCouponType>((a, b) => a.CTypeId == b.Id)
                .Where((a,b)=>a.GroupId== groupId)
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.CTypeId), a => a.CTypeId == inputDto.CTypeId)
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.Title), a => a.Title.Contains(inputDto.Title))
                .WhereIF(inputDto.Money > 0, a => a.Money == inputDto.Money)
                .WhereIF(inputDto.Enable!=null,a=>a.Enable== inputDto.Enable)
                .Select((a,b)=>new PaperCouponBasetDto()
                {
                    Id=a.Id, CTypeId=a.CTypeId, Title=a.Title,Money=a.Money,CTypeName=b.Name,CrateTimeAt=a.CrateTimeAt,Enable=a.Enable,SoureMoney=a.SoureMoney
                }).ToPagedListAsync(inputDto.Current, inputDto.PageSize);
            result = queryable.Items.ToList();
            return new PageUtil
            {
                Total = queryable.TotalCount,
                Data = result
            };
        }

        /// <summary>
        /// 新增优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> AddPaperCoupon(PaperCouponBasetDto inputDto)
        {
            if (string.IsNullOrWhiteSpace(inputDto.Title))
            {
                throw Oops.Oh("标题参数不能为空");
            }
            if (string.IsNullOrWhiteSpace(inputDto.CTypeId))
            {
                throw Oops.Oh("类型参数不能为空");
            }
            if (inputDto.Money==null)
            {
                throw Oops.Oh("金额参数不能为空");
            }
            var checkMoneyType=db.Queryable<BPA_PaperCoupon>().Any(x => x.CTypeId == inputDto.CTypeId && x.Money == inputDto.Money);
            if(checkMoneyType) throw Oops.Oh("已存在相同类型金额的优惠卷");
            inputDto.Id = Guid.NewGuid().ToString();
            inputDto.CrateTimeAt = DateTime.Now;
            inputDto.Enable= true;
            var result = await db.Insertable<BPA_PaperCoupon>(inputDto.Adapt<BPA_PaperCoupon>()).ExecuteCommandAsync();

            return await Task.FromResult(result > 0);
        }

        /// <summary>
        /// 删除优惠卷
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        public async Task<bool> UpdatePaperCouponEnable(string Ids)
        {
            var model = db.Queryable<BPA_PaperCoupon>().First(x => Ids.Contains(x.Id));
            if (model == null) throw Oops.Oh("数据不存在");
            model.Enable = !model.Enable;
            var result = db.Updateable<BPA_PaperCoupon>(model).ExecuteCommand();
            return result > 0;
        }

        /// <summary>
        /// 修改优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdatePaperCoupon([FromBody] PaperCouponBasetDto inputDto)
        {
            if (string.IsNullOrWhiteSpace(inputDto.Title))
            {
                throw Oops.Oh("标题参数不能为空");
            }
            if (string.IsNullOrWhiteSpace(inputDto.CTypeId))
            {
                throw Oops.Oh("类型参数不能为空");
            }
            if (inputDto.Money == null)
            {
                throw Oops.Oh("金额参数不能为空");
            }
            if (inputDto.Money<0)
            {
                throw Oops.Oh("金额不能小于0");
            }
			if (inputDto.SoureMoney < 0)
			{
				throw Oops.Oh("金额不能小于0");
			}
			var result = db.Updateable<BPA_PaperCoupon>(inputDto.Adapt<BPA_PaperCoupon>()).UpdateColumns(it =>
                new { it.Title,it.Money,it.CTypeId }).ExecuteCommand();
           return await Task.FromResult(result > 0);
        }
        /// <summary>
        /// 新增优惠卷
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> AddPaperCouponType(PaperCouponTypeBasetDto inputDto)
        {
            if (string.IsNullOrWhiteSpace(inputDto.Name))
            {
                throw Oops.Oh("标题参数不能为空");
            }
            var checkMoneyType = db.Queryable<BPA_PaperCouponType>().Any(x => x.Name == inputDto.Name);
            if (checkMoneyType) throw Oops.Oh("已存在相同名称的类型");
            inputDto.Id = Guid.NewGuid().ToString();
            var result = await db.Insertable<BPA_PaperCouponType>(inputDto.Adapt<BPA_PaperCouponType>()).ExecuteCommandAsync();

            return await Task.FromResult(result > 0);
        }
        /// <summary>
        /// 查询所有优惠卷类型
        /// </summary>
        /// <returns></returns>
        public async Task<List<PaperCouponTypeSelectDto>> GetPaperCouponType()
        {
            var list= await db.Queryable<BPA_PaperCouponType>().Select(x=>new PaperCouponTypeSelectDto()
            {
                Key=x.Id,Value=x.Name
            }).ToListAsync();
            return list;
        }
        public async Task<List<PaperCouponReportDatadto>> GetPaperCouponReport(PaperCouponReportdto paperCouponReportdto)
        {
            var groupId = string.IsNullOrWhiteSpace(CurrentUser.TenantId)
               ? App.User?.FindFirst(ClaimConst.GroupId)?.Value
               : CurrentUser.TenantId;
            var queryable = await db.Queryable<BPA_PaperCoupon, BPA_PaperCouponType>((a, b) => a.CTypeId == b.Id)
               .WhereIF(!string.IsNullOrWhiteSpace(paperCouponReportdto.CTypeId), a => a.CTypeId == paperCouponReportdto.CTypeId)
               .WhereIF(paperCouponReportdto.Money > 0, a => a.Money == paperCouponReportdto.Money)
              
               .Select((a, b) => new PaperCouponReportDatadto()
               {
                   Id = a.Id,
                   CTypeId = a.CTypeId,
                   Title = a.Title,
                   Money = a.Money,
                   SoureMoney= a.SoureMoney,
                   CTypeName = b.Name,
                   CrateTimeAt = a.CrateTimeAt
               }).Mapper(p =>
               {
                   p.Allcount= db.Queryable<BPA_PaperCouponWriteoffLog>()
                    .Where(x => x.PaperCouponId == p.Id && x.GroupId== groupId)
                   .WhereIF(!string.IsNullOrWhiteSpace(paperCouponReportdto.StartDateStr),c=>c.CreateAt>=Convert.ToDateTime(paperCouponReportdto.StartDateStr))
                   .WhereIF(!string.IsNullOrWhiteSpace(paperCouponReportdto.EndDateStr), c => c.CreateAt <=Convert.ToDateTime(paperCouponReportdto.EndDateStr))
                   .Count();
               })
               .ToListAsync();
            return queryable;
        }
    }
}
