﻿using Furion;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using BPA.KitChen.StoreManagement.Core.Common.Const;
using BPA.KitChen.StoreManagement.Core.Entity.Base;

namespace BPA.KitChen.StoreManagement.Core.Entity.Coupon
{
    /// <summary>
    /// 顾客优惠券
    /// </summary>
    [SugarTable("BPA_CustomerCoupon")]
    public class BPA_CustomerCoupon: IBaseGroupIdEntity
    {
        /// <summary>
        /// 客户Id
        /// </summary>
        [SugarColumn(ColumnDataType = "Nvarchar(64)",IsNullable =false)]
        public string Customer_Id { get; set; }
        /// <summary>
        /// 订单Id
        /// </summary>
        [SugarColumn(ColumnDataType = "Nvarchar(64)", IsNullable = true)]
        public string Order_Id { get; set; }
        /// <summary>
        /// 领取方式(1.小程序领取 2.线下门店扫码 3.活动自动下发  4.好友分享领取 5.指定发放)
        /// </summary>
        [SugarColumn(ColumnDataType = "INT", IsNullable = false,ColumnName = "ReceiveType")]
        public int Receive_Type { get; set; }
        /// <summary>
        /// 优惠券Id
        /// </summary>
        [SugarColumn(ColumnDataType = "Nvarchar(64)", IsNullable = false)]
        public string Coupon_Id { get; set; }
        /// <summary>
        /// 状态（1:未使用 2.已核销 3.占用中 4.失效）
        /// </summary>
        [SugarColumn(ColumnDataType = "INT", IsNullable = true)]
        public int State { get; set; }
        /// <summary>
        /// 开始时间
        /// </summary>
        [SugarColumn(ColumnDataType = "DATETIME", IsNullable = false,ColumnName = "ValidStartTime")]
        public DateTime Valid_StartTime { get; set; }
        
        /// <summary>
        /// 结束时间
        /// </summary>
        [SugarColumn(ColumnDataType = "DATETIME", IsNullable = false, ColumnName = "ValidEndTime")]
        public DateTime Valid_EndTime { get; set; }
        /// <summary>
        /// 核销时间
        /// </summary>
        [SugarColumn(ColumnDataType = "DATETIME", IsNullable = true)]
        public DateTime? WriteOffTime { get; set; }
        /// <summary>
        /// 范围Id
        /// </summary>
        [SugarColumn(ColumnDataType = "Nvarchar(64)", IsNullable = false)]
        public string Coupon_Range_Id { get; set; }
        /// <summary>
        /// 用户手机号
        /// </summary>
        [SugarColumn(ColumnDataType = "Nvarchar(64)", IsNullable = true)]
        public string CustomerPhone { get; set; }

        /// <summary>
        /// 新增
        /// </summary>
        public  void Create()
        {
            var userId = App.User.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            this.Id = Guid.NewGuid().ToString();
            if (!string.IsNullOrEmpty(userId))
            {
                this.CreateBy = userId;
            }
        }
        /// <summary>
        /// 修改
        /// </summary>
        public void Modify()
        {
            var userId = App.User.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            this.UpdateAt = DateTime.Now;
            if (!string.IsNullOrEmpty(userId))
            {
                this.UpdateBy = userId;
            }
        }

        /// <summary>
        /// 删除
        /// </summary>
        public void Delete()
        {
            var userId = App.User.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            this.DeleteAt = DateTime.Now;
            if (!string.IsNullOrEmpty(userId))
            {
                this.DeleteBy = userId;
            }
        }

    }
}
